# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..models import common
from ..cog import runtime as cogruntime


class Options:
    # Size of each bucket
    bucket_size: typing.Optional[int]
    # Offset buckets by this amount
    bucket_offset: typing.Optional[int]
    legend: common.VizLegendOptions
    tooltip: common.VizTooltipOptions
    # Combines multiple series into a single histogram
    combine: typing.Optional[bool]

    def __init__(self, bucket_size: typing.Optional[int] = None, bucket_offset: typing.Optional[int] = 0, legend: typing.Optional[common.VizLegendOptions] = None, tooltip: typing.Optional[common.VizTooltipOptions] = None, combine: typing.Optional[bool] = None):
        self.bucket_size = bucket_size
        self.bucket_offset = bucket_offset
        self.legend = legend if legend is not None else common.VizLegendOptions()
        self.tooltip = tooltip if tooltip is not None else common.VizTooltipOptions()
        self.combine = combine

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "legend": self.legend,
            "tooltip": self.tooltip,
        }
        if self.bucket_size is not None:
            payload["bucketSize"] = self.bucket_size
        if self.bucket_offset is not None:
            payload["bucketOffset"] = self.bucket_offset
        if self.combine is not None:
            payload["combine"] = self.combine
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "bucketSize" in data:
            args["bucket_size"] = data["bucketSize"]
        if "bucketOffset" in data:
            args["bucket_offset"] = data["bucketOffset"]
        if "legend" in data:
            args["legend"] = common.VizLegendOptions.from_json(data["legend"])
        if "tooltip" in data:
            args["tooltip"] = common.VizTooltipOptions.from_json(data["tooltip"])
        if "combine" in data:
            args["combine"] = data["combine"]        

        return cls(**args)


class FieldConfig:
    # Controls line width of the bars.
    line_width: typing.Optional[int]
    # Controls the fill opacity of the bars.
    fill_opacity: typing.Optional[int]
    axis_placement: typing.Optional[common.AxisPlacement]
    axis_color_mode: typing.Optional[common.AxisColorMode]
    axis_label: typing.Optional[str]
    axis_width: typing.Optional[float]
    axis_soft_min: typing.Optional[float]
    axis_soft_max: typing.Optional[float]
    axis_grid_show: typing.Optional[bool]
    scale_distribution: typing.Optional[common.ScaleDistributionConfig]
    axis_centered_zero: typing.Optional[bool]
    hide_from: typing.Optional[common.HideSeriesConfig]
    # Set the mode of the gradient fill. Fill gradient is based on the line color. To change the color, use the standard color scheme field option.
    # Gradient appearance is influenced by the Fill opacity setting.
    gradient_mode: typing.Optional[common.GraphGradientMode]
    axis_border_show: typing.Optional[bool]

    def __init__(self, line_width: typing.Optional[int] = 1, fill_opacity: typing.Optional[int] = 80, axis_placement: typing.Optional[common.AxisPlacement] = None, axis_color_mode: typing.Optional[common.AxisColorMode] = None, axis_label: typing.Optional[str] = None, axis_width: typing.Optional[float] = None, axis_soft_min: typing.Optional[float] = None, axis_soft_max: typing.Optional[float] = None, axis_grid_show: typing.Optional[bool] = None, scale_distribution: typing.Optional[common.ScaleDistributionConfig] = None, axis_centered_zero: typing.Optional[bool] = None, hide_from: typing.Optional[common.HideSeriesConfig] = None, gradient_mode: typing.Optional[common.GraphGradientMode] = None, axis_border_show: typing.Optional[bool] = None):
        self.line_width = line_width
        self.fill_opacity = fill_opacity
        self.axis_placement = axis_placement
        self.axis_color_mode = axis_color_mode
        self.axis_label = axis_label
        self.axis_width = axis_width
        self.axis_soft_min = axis_soft_min
        self.axis_soft_max = axis_soft_max
        self.axis_grid_show = axis_grid_show
        self.scale_distribution = scale_distribution
        self.axis_centered_zero = axis_centered_zero
        self.hide_from = hide_from
        self.gradient_mode = gradient_mode if gradient_mode is not None else common.GraphGradientMode.NONE
        self.axis_border_show = axis_border_show

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.line_width is not None:
            payload["lineWidth"] = self.line_width
        if self.fill_opacity is not None:
            payload["fillOpacity"] = self.fill_opacity
        if self.axis_placement is not None:
            payload["axisPlacement"] = self.axis_placement
        if self.axis_color_mode is not None:
            payload["axisColorMode"] = self.axis_color_mode
        if self.axis_label is not None:
            payload["axisLabel"] = self.axis_label
        if self.axis_width is not None:
            payload["axisWidth"] = self.axis_width
        if self.axis_soft_min is not None:
            payload["axisSoftMin"] = self.axis_soft_min
        if self.axis_soft_max is not None:
            payload["axisSoftMax"] = self.axis_soft_max
        if self.axis_grid_show is not None:
            payload["axisGridShow"] = self.axis_grid_show
        if self.scale_distribution is not None:
            payload["scaleDistribution"] = self.scale_distribution
        if self.axis_centered_zero is not None:
            payload["axisCenteredZero"] = self.axis_centered_zero
        if self.hide_from is not None:
            payload["hideFrom"] = self.hide_from
        if self.gradient_mode is not None:
            payload["gradientMode"] = self.gradient_mode
        if self.axis_border_show is not None:
            payload["axisBorderShow"] = self.axis_border_show
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "lineWidth" in data:
            args["line_width"] = data["lineWidth"]
        if "fillOpacity" in data:
            args["fill_opacity"] = data["fillOpacity"]
        if "axisPlacement" in data:
            args["axis_placement"] = data["axisPlacement"]
        if "axisColorMode" in data:
            args["axis_color_mode"] = data["axisColorMode"]
        if "axisLabel" in data:
            args["axis_label"] = data["axisLabel"]
        if "axisWidth" in data:
            args["axis_width"] = data["axisWidth"]
        if "axisSoftMin" in data:
            args["axis_soft_min"] = data["axisSoftMin"]
        if "axisSoftMax" in data:
            args["axis_soft_max"] = data["axisSoftMax"]
        if "axisGridShow" in data:
            args["axis_grid_show"] = data["axisGridShow"]
        if "scaleDistribution" in data:
            args["scale_distribution"] = common.ScaleDistributionConfig.from_json(data["scaleDistribution"])
        if "axisCenteredZero" in data:
            args["axis_centered_zero"] = data["axisCenteredZero"]
        if "hideFrom" in data:
            args["hide_from"] = common.HideSeriesConfig.from_json(data["hideFrom"])
        if "gradientMode" in data:
            args["gradient_mode"] = data["gradientMode"]
        if "axisBorderShow" in data:
            args["axis_border_show"] = data["axisBorderShow"]        

        return cls(**args)





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="histogram",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=FieldConfig.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.


from ..models import alertgroups
from ..models import annotationslist
from ..models import barchart
from ..models import bargauge
from ..models import candlestick
from ..models import canvas
from ..models import dashboardlist
from ..models import datagrid
from ..models import debug
from ..models import gauge
from ..models import geomap
from ..models import heatmap
from ..models import histogram
from ..models import logs
from ..models import news
from ..models import nodegraph
from ..models import piechart
from ..models import stat
from ..models import statetimeline
from ..models import statushistory
from ..models import table
from ..models import text
from ..models import timeseries
from ..models import trend
from ..models import xychart
from ..models import athena
from ..models import azuremonitor
from ..models import bigquery
from ..models import cloudwatch
from ..models import datasource
from ..models import elasticsearch
from ..models import expr
from ..models import googlecloudmonitoring
from ..models import grafanapyroscope
from ..models import loki
from ..models import parca
from ..models import prometheus
from ..models import tempo
from ..models import testdata
from . import runtime as cogruntime


def register_default_plugins():
    # Panelcfg variants
    cogruntime.register_panelcfg_variant(alertgroups.variant_config())
    cogruntime.register_panelcfg_variant(annotationslist.variant_config())
    cogruntime.register_panelcfg_variant(barchart.variant_config())
    cogruntime.register_panelcfg_variant(bargauge.variant_config())
    cogruntime.register_panelcfg_variant(candlestick.variant_config())
    cogruntime.register_panelcfg_variant(canvas.variant_config())
    cogruntime.register_panelcfg_variant(dashboardlist.variant_config())
    cogruntime.register_panelcfg_variant(datagrid.variant_config())
    cogruntime.register_panelcfg_variant(debug.variant_config())
    cogruntime.register_panelcfg_variant(gauge.variant_config())
    cogruntime.register_panelcfg_variant(geomap.variant_config())
    cogruntime.register_panelcfg_variant(heatmap.variant_config())
    cogruntime.register_panelcfg_variant(histogram.variant_config())
    cogruntime.register_panelcfg_variant(logs.variant_config())
    cogruntime.register_panelcfg_variant(news.variant_config())
    cogruntime.register_panelcfg_variant(nodegraph.variant_config())
    cogruntime.register_panelcfg_variant(piechart.variant_config())
    cogruntime.register_panelcfg_variant(stat.variant_config())
    cogruntime.register_panelcfg_variant(statetimeline.variant_config())
    cogruntime.register_panelcfg_variant(statushistory.variant_config())
    cogruntime.register_panelcfg_variant(table.variant_config())
    cogruntime.register_panelcfg_variant(text.variant_config())
    cogruntime.register_panelcfg_variant(timeseries.variant_config())
    cogruntime.register_panelcfg_variant(trend.variant_config())
    cogruntime.register_panelcfg_variant(xychart.variant_config())

    # Dataquery variants
    cogruntime.register_dataquery_variant(athena.variant_config())
    cogruntime.register_dataquery_variant(azuremonitor.variant_config())
    cogruntime.register_dataquery_variant(bigquery.variant_config())
    cogruntime.register_dataquery_variant(cloudwatch.variant_config())
    cogruntime.register_dataquery_variant(datasource.variant_config())
    cogruntime.register_dataquery_variant(elasticsearch.variant_config())
    cogruntime.register_dataquery_variant(expr.variant_config())
    cogruntime.register_dataquery_variant(googlecloudmonitoring.variant_config())
    cogruntime.register_dataquery_variant(grafanapyroscope.variant_config())
    cogruntime.register_dataquery_variant(loki.variant_config())
    cogruntime.register_dataquery_variant(parca.variant_config())
    cogruntime.register_dataquery_variant(prometheus.variant_config())
    cogruntime.register_dataquery_variant(tempo.variant_config())
    cogruntime.register_dataquery_variant(testdata.variant_config())



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..models import common
import typing
from ..cog import runtime as cogruntime


class Options:
    graph_mode: common.BigValueGraphMode
    color_mode: common.BigValueColorMode
    justify_mode: common.BigValueJustifyMode
    text_mode: common.BigValueTextMode
    reduce_options: common.ReduceDataOptions
    text: typing.Optional[common.VizTextDisplayOptions]
    wide_layout: bool
    orientation: common.VizOrientation

    def __init__(self, graph_mode: typing.Optional[common.BigValueGraphMode] = None, color_mode: typing.Optional[common.BigValueColorMode] = None, justify_mode: typing.Optional[common.BigValueJustifyMode] = None, text_mode: typing.Optional[common.BigValueTextMode] = None, reduce_options: typing.Optional[common.ReduceDataOptions] = None, text: typing.Optional[common.VizTextDisplayOptions] = None, wide_layout: bool = True, orientation: typing.Optional[common.VizOrientation] = None):
        self.graph_mode = graph_mode if graph_mode is not None else common.BigValueGraphMode.AREA
        self.color_mode = color_mode if color_mode is not None else common.BigValueColorMode.VALUE
        self.justify_mode = justify_mode if justify_mode is not None else common.BigValueJustifyMode.AUTO
        self.text_mode = text_mode if text_mode is not None else common.BigValueTextMode.AUTO
        self.reduce_options = reduce_options if reduce_options is not None else common.ReduceDataOptions()
        self.text = text
        self.wide_layout = wide_layout
        self.orientation = orientation if orientation is not None else common.VizOrientation.AUTO

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "graphMode": self.graph_mode,
            "colorMode": self.color_mode,
            "justifyMode": self.justify_mode,
            "textMode": self.text_mode,
            "reduceOptions": self.reduce_options,
            "wideLayout": self.wide_layout,
            "orientation": self.orientation,
        }
        if self.text is not None:
            payload["text"] = self.text
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "graphMode" in data:
            args["graph_mode"] = data["graphMode"]
        if "colorMode" in data:
            args["color_mode"] = data["colorMode"]
        if "justifyMode" in data:
            args["justify_mode"] = data["justifyMode"]
        if "textMode" in data:
            args["text_mode"] = data["textMode"]
        if "reduceOptions" in data:
            args["reduce_options"] = common.ReduceDataOptions.from_json(data["reduceOptions"])
        if "text" in data:
            args["text"] = common.VizTextDisplayOptions.from_json(data["text"])
        if "wideLayout" in data:
            args["wide_layout"] = data["wideLayout"]
        if "orientation" in data:
            args["orientation"] = data["orientation"]        

        return cls(**args)


def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="stat",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from json import JSONEncoder as BaseJSONEncoder


class JSONEncoder(BaseJSONEncoder):
    def default(self, obj):
        obj_to_json = getattr(obj, "to_json", None)
        if callable(obj_to_json):
            return obj_to_json()

        return BaseJSONEncoder.default(self, obj)



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import athena
from ..models import dashboard


class Dataquery(cogbuilder.Builder[athena.Dataquery]):    
    """
    Manually converted from https://github.com/grafana/athena-datasource/blob/57ad707147b7a11e9a521a836d6bf9799877e0e3/src/types.ts
    """
    
    _internal: athena.Dataquery

    def __init__(self):
        self._internal = athena.Dataquery()

    def build(self) -> athena.Dataquery:
        """
        Builds the object.
        """
        return self._internal    
    
    def format(self, format_val: athena.FormatOptions) -> typing.Self:    
        self._internal.format_val = format_val
    
        return self
    
    def connection_args(self, connection_args: cogbuilder.Builder[athena.ConnectionArgs]) -> typing.Self:    
        connection_args_resource = connection_args.build()
        self._internal.connection_args = connection_args_resource
    
        return self
    
    def table(self, table: str) -> typing.Self:    
        self._internal.table = table
    
        return self
    
    def column(self, column: str) -> typing.Self:    
        self._internal.column = column
    
        return self
    
    def query_id(self, query_id: str) -> typing.Self:    
        self._internal.query_id = query_id
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def raw_sql(self, raw_sql: str) -> typing.Self:    
        self._internal.raw_sql = raw_sql
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    


class ConnectionArgs(cogbuilder.Builder[athena.ConnectionArgs]):
    _internal: athena.ConnectionArgs

    def __init__(self):
        self._internal = athena.ConnectionArgs()

    def build(self) -> athena.ConnectionArgs:
        """
        Builds the object.
        """
        return self._internal    
    
    def region(self, region: str) -> typing.Self:    
        self._internal.region = region
    
        return self
    
    def catalog(self, catalog: str) -> typing.Self:    
        self._internal.catalog = catalog
    
        return self
    
    def database(self, database: str) -> typing.Self:    
        self._internal.database = database
    
        return self
    
    def result_reuse_enabled(self, result_reuse_enabled: bool) -> typing.Self:    
        self._internal.result_reuse_enabled = result_reuse_enabled
    
        return self
    
    def result_reuse_max_age_in_minutes(self, result_reuse_max_age_in_minutes: float) -> typing.Self:    
        self._internal.result_reuse_max_age_in_minutes = result_reuse_max_age_in_minutes
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import variants as cogvariants
from ..models import dashboard
from ..cog import runtime as cogruntime


class LibraryPanel:
    # Folder UID
    folder_uid: typing.Optional[str]
    # Library element UID
    uid: str
    # Panel name (also saved in the model)
    name: str
    # Panel description
    description: typing.Optional[str]
    # The panel type (from inside the model)
    type_val: str
    # Dashboard version when this was saved (zero if unknown)
    schema_version: typing.Optional[int]
    # panel version, incremented each time the dashboard is updated.
    version: int
    # TODO: should be the same panel schema defined in dashboard
    # Typescript: Omit<Panel, 'gridPos' | 'id' | 'libraryPanel'>;
    model: 'PanelModel'
    # Object storage metadata
    meta: typing.Optional['LibraryElementDTOMeta']

    def __init__(self, folder_uid: typing.Optional[str] = None, uid: str = "", name: str = "", description: typing.Optional[str] = None, type_val: str = "", schema_version: typing.Optional[int] = None, version: int = 0, model: typing.Optional['PanelModel'] = None, meta: typing.Optional['LibraryElementDTOMeta'] = None):
        self.folder_uid = folder_uid
        self.uid = uid
        self.name = name
        self.description = description
        self.type_val = type_val
        self.schema_version = schema_version
        self.version = version
        self.model = model if model is not None else PanelModel()
        self.meta = meta

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "uid": self.uid,
            "name": self.name,
            "type": self.type_val,
            "version": self.version,
            "model": self.model,
        }
        if self.folder_uid is not None:
            payload["folderUid"] = self.folder_uid
        if self.description is not None:
            payload["description"] = self.description
        if self.schema_version is not None:
            payload["schemaVersion"] = self.schema_version
        if self.meta is not None:
            payload["meta"] = self.meta
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "folderUid" in data:
            args["folder_uid"] = data["folderUid"]
        if "uid" in data:
            args["uid"] = data["uid"]
        if "name" in data:
            args["name"] = data["name"]
        if "description" in data:
            args["description"] = data["description"]
        if "type" in data:
            args["type_val"] = data["type"]
        if "schemaVersion" in data:
            args["schema_version"] = data["schemaVersion"]
        if "version" in data:
            args["version"] = data["version"]
        if "model" in data:
            args["model"] = PanelModel.from_json(data["model"])
        if "meta" in data:
            args["meta"] = LibraryElementDTOMeta.from_json(data["meta"])        

        return cls(**args)


class LibraryElementDTOMeta:
    folder_name: str
    folder_uid: str
    connected_dashboards: int
    created: str
    updated: str
    created_by: 'LibraryElementDTOMetaUser'
    updated_by: 'LibraryElementDTOMetaUser'

    def __init__(self, folder_name: str = "", folder_uid: str = "", connected_dashboards: int = 0, created: str = "", updated: str = "", created_by: typing.Optional['LibraryElementDTOMetaUser'] = None, updated_by: typing.Optional['LibraryElementDTOMetaUser'] = None):
        self.folder_name = folder_name
        self.folder_uid = folder_uid
        self.connected_dashboards = connected_dashboards
        self.created = created
        self.updated = updated
        self.created_by = created_by if created_by is not None else LibraryElementDTOMetaUser()
        self.updated_by = updated_by if updated_by is not None else LibraryElementDTOMetaUser()

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "folderName": self.folder_name,
            "folderUid": self.folder_uid,
            "connectedDashboards": self.connected_dashboards,
            "created": self.created,
            "updated": self.updated,
            "createdBy": self.created_by,
            "updatedBy": self.updated_by,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "folderName" in data:
            args["folder_name"] = data["folderName"]
        if "folderUid" in data:
            args["folder_uid"] = data["folderUid"]
        if "connectedDashboards" in data:
            args["connected_dashboards"] = data["connectedDashboards"]
        if "created" in data:
            args["created"] = data["created"]
        if "updated" in data:
            args["updated"] = data["updated"]
        if "createdBy" in data:
            args["created_by"] = LibraryElementDTOMetaUser.from_json(data["createdBy"])
        if "updatedBy" in data:
            args["updated_by"] = LibraryElementDTOMetaUser.from_json(data["updatedBy"])        

        return cls(**args)


class LibraryElementDTOMetaUser:
    id_val: int
    name: str
    avatar_url: str

    def __init__(self, id_val: int = 0, name: str = "", avatar_url: str = ""):
        self.id_val = id_val
        self.name = name
        self.avatar_url = avatar_url

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "id": self.id_val,
            "name": self.name,
            "avatarUrl": self.avatar_url,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "id" in data:
            args["id_val"] = data["id"]
        if "name" in data:
            args["name"] = data["name"]
        if "avatarUrl" in data:
            args["avatar_url"] = data["avatarUrl"]        

        return cls(**args)


class PanelModel:
    """
    Dashboard panels are the basic visualization building blocks.
    """

    # The panel plugin type id. This is used to find the plugin to display the panel.
    type_val: str
    # The version of the plugin that is used for this panel. This is used to find the plugin to display the panel and to migrate old panel configs.
    plugin_version: typing.Optional[str]
    # Tags for the panel.
    tags: typing.Optional[list[str]]
    # Depends on the panel plugin. See the plugin documentation for details.
    targets: typing.Optional[list[cogvariants.Dataquery]]
    # Panel title.
    title: typing.Optional[str]
    # Panel description.
    description: typing.Optional[str]
    # Whether to display the panel without a background.
    transparent: typing.Optional[bool]
    # The datasource used in all targets.
    datasource: typing.Optional[dashboard.DataSourceRef]
    # Panel links.
    links: typing.Optional[list[dashboard.DashboardLink]]
    # Name of template variable to repeat for.
    repeat: typing.Optional[str]
    # Direction to repeat in if 'repeat' is set.
    # `h` for horizontal, `v` for vertical.
    repeat_direction: typing.Optional[typing.Literal["h", "v"]]
    # Option for repeated panels that controls max items per row
    # Only relevant for horizontally repeated panels
    max_per_row: typing.Optional[float]
    # The maximum number of data points that the panel queries are retrieving.
    max_data_points: typing.Optional[float]
    # List of transformations that are applied to the panel data before rendering.
    # When there are multiple transformations, Grafana applies them in the order they are listed.
    # Each transformation creates a result set that then passes on to the next transformation in the processing pipeline.
    transformations: typing.Optional[list[dashboard.DataTransformerConfig]]
    # The min time interval setting defines a lower limit for the $__interval and $__interval_ms variables.
    # This value must be formatted as a number followed by a valid time
    # identifier like: "40s", "3d", etc.
    # See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
    interval: typing.Optional[str]
    # Overrides the relative time range for individual panels,
    # which causes them to be different than what is selected in
    # the dashboard time picker in the top-right corner of the dashboard. You can use this to show metrics from different
    # time periods or days on the same dashboard.
    # The value is formatted as time operation like: `now-5m` (Last 5 minutes), `now/d` (the day so far),
    # `now-5d/d`(Last 5 days), `now/w` (This week so far), `now-2y/y` (Last 2 years).
    # Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
    # See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
    time_from: typing.Optional[str]
    # Overrides the time range for individual panels by shifting its start and end relative to the time picker.
    # For example, you can shift the time range for the panel to be two hours earlier than the dashboard time picker setting `2h`.
    # Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
    # See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
    time_shift: typing.Optional[str]
    # Controls if the timeFrom or timeShift overrides are shown in the panel header
    hide_time_override: typing.Optional[bool]
    # It depends on the panel plugin. They are specified by the Options field in panel plugin schemas.
    options: typing.Optional[object]
    # Field options allow you to change how the data is displayed in your visualizations.
    field_config: typing.Optional[dashboard.FieldConfigSource]

    def __init__(self, type_val: str = "", plugin_version: typing.Optional[str] = None, tags: typing.Optional[list[str]] = None, targets: typing.Optional[list[cogvariants.Dataquery]] = None, title: typing.Optional[str] = None, description: typing.Optional[str] = None, transparent: typing.Optional[bool] = False, datasource: typing.Optional[dashboard.DataSourceRef] = None, links: typing.Optional[list[dashboard.DashboardLink]] = None, repeat: typing.Optional[str] = None, repeat_direction: typing.Optional[typing.Literal["h", "v"]] = None, max_per_row: typing.Optional[float] = None, max_data_points: typing.Optional[float] = None, transformations: typing.Optional[list[dashboard.DataTransformerConfig]] = None, interval: typing.Optional[str] = None, time_from: typing.Optional[str] = None, time_shift: typing.Optional[str] = None, hide_time_override: typing.Optional[bool] = None, options: typing.Optional[object] = None, field_config: typing.Optional[dashboard.FieldConfigSource] = None):
        self.type_val = type_val
        self.plugin_version = plugin_version
        self.tags = tags
        self.targets = targets
        self.title = title
        self.description = description
        self.transparent = transparent
        self.datasource = datasource
        self.links = links
        self.repeat = repeat
        self.repeat_direction = repeat_direction if repeat_direction is not None else "h"
        self.max_per_row = max_per_row
        self.max_data_points = max_data_points
        self.transformations = transformations
        self.interval = interval
        self.time_from = time_from
        self.time_shift = time_shift
        self.hide_time_override = hide_time_override
        self.options = options
        self.field_config = field_config

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "type": self.type_val,
        }
        if self.plugin_version is not None:
            payload["pluginVersion"] = self.plugin_version
        if self.tags is not None:
            payload["tags"] = self.tags
        if self.targets is not None:
            payload["targets"] = self.targets
        if self.title is not None:
            payload["title"] = self.title
        if self.description is not None:
            payload["description"] = self.description
        if self.transparent is not None:
            payload["transparent"] = self.transparent
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.links is not None:
            payload["links"] = self.links
        if self.repeat is not None:
            payload["repeat"] = self.repeat
        if self.repeat_direction is not None:
            payload["repeatDirection"] = self.repeat_direction
        if self.max_per_row is not None:
            payload["maxPerRow"] = self.max_per_row
        if self.max_data_points is not None:
            payload["maxDataPoints"] = self.max_data_points
        if self.transformations is not None:
            payload["transformations"] = self.transformations
        if self.interval is not None:
            payload["interval"] = self.interval
        if self.time_from is not None:
            payload["timeFrom"] = self.time_from
        if self.time_shift is not None:
            payload["timeShift"] = self.time_shift
        if self.hide_time_override is not None:
            payload["hideTimeOverride"] = self.hide_time_override
        if self.options is not None:
            payload["options"] = self.options
        if self.field_config is not None:
            payload["fieldConfig"] = self.field_config
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "type" in data:
            args["type_val"] = data["type"]
        if "pluginVersion" in data:
            args["plugin_version"] = data["pluginVersion"]
        if "tags" in data:
            args["tags"] = data["tags"]
        if "targets" in data:
            args["targets"] = [cogruntime.dataquery_from_json(dataquery_json, data["datasource"]["type"] if data.get("datasource") is not None and data["datasource"].get("type", "") != "" else "") for dataquery_json in data["targets"]]
        if "title" in data:
            args["title"] = data["title"]
        if "description" in data:
            args["description"] = data["description"]
        if "transparent" in data:
            args["transparent"] = data["transparent"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "links" in data:
            args["links"] = [dashboard.DashboardLink.from_json(item) for item in data["links"]]
        if "repeat" in data:
            args["repeat"] = data["repeat"]
        if "repeatDirection" in data:
            args["repeat_direction"] = data["repeatDirection"]
        if "maxPerRow" in data:
            args["max_per_row"] = data["maxPerRow"]
        if "maxDataPoints" in data:
            args["max_data_points"] = data["maxDataPoints"]
        if "transformations" in data:
            args["transformations"] = [dashboard.DataTransformerConfig.from_json(item) for item in data["transformations"]]
        if "interval" in data:
            args["interval"] = data["interval"]
        if "timeFrom" in data:
            args["time_from"] = data["timeFrom"]
        if "timeShift" in data:
            args["time_shift"] = data["timeShift"]
        if "hideTimeOverride" in data:
            args["hide_time_override"] = data["hideTimeOverride"]
        if "options" in data:
            args["options"] = data["options"]
        if "fieldConfig" in data:
            args["field_config"] = dashboard.FieldConfigSource.from_json(data["fieldConfig"])        

        return cls(**args)






# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
from ..cog import variants as cogvariants
import typing
from ..models import dashboard
from ..cog import runtime as cogruntime


class ParcaQueryType(enum.StrEnum):
    METRICS = "metrics"
    PROFILE = "profile"
    BOTH = "both"


class Dataquery(cogvariants.Dataquery):
    # Specifies the query label selectors.
    label_selector: str
    # Specifies the type of profile to query.
    profile_type_id: str
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]

    def __init__(self, label_selector: str = "{}", profile_type_id: str = "", ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None):
        self.label_selector = label_selector
        self.profile_type_id = profile_type_id
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.datasource = datasource

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "labelSelector": self.label_selector,
            "profileTypeId": self.profile_type_id,
            "refId": self.ref_id,
        }
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "labelSelector" in data:
            args["label_selector"] = data["labelSelector"]
        if "profileTypeId" in data:
            args["profile_type_id"] = data["profileTypeId"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])        

        return cls(**args)





def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="parca",
        from_json_hook=Dataquery.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing


class Role:
    # The role identifier `managed:builtins:editor:permissions`
    name: str
    # Optional display
    display_name: typing.Optional[str]
    # Name of the team.
    group_name: typing.Optional[str]
    # Role description
    description: typing.Optional[str]
    # Do not show this role
    hidden: bool

    def __init__(self, name: str = "", display_name: typing.Optional[str] = None, group_name: typing.Optional[str] = None, description: typing.Optional[str] = None, hidden: bool = False):
        self.name = name
        self.display_name = display_name
        self.group_name = group_name
        self.description = description
        self.hidden = hidden

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "name": self.name,
            "hidden": self.hidden,
        }
        if self.display_name is not None:
            payload["displayName"] = self.display_name
        if self.group_name is not None:
            payload["groupName"] = self.group_name
        if self.description is not None:
            payload["description"] = self.description
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "name" in data:
            args["name"] = data["name"]
        if "displayName" in data:
            args["display_name"] = data["displayName"]
        if "groupName" in data:
            args["group_name"] = data["groupName"]
        if "description" in data:
            args["description"] = data["description"]
        if "hidden" in data:
            args["hidden"] = data["hidden"]        

        return cls(**args)



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
import typing
from ..cog import runtime as cogruntime


class TextMode(enum.StrEnum):
    HTML = "html"
    MARKDOWN = "markdown"
    CODE = "code"


class CodeLanguage(enum.StrEnum):
    JSON = "json"
    YAML = "yaml"
    XML = "xml"
    TYPESCRIPT = "typescript"
    SQL = "sql"
    GO = "go"
    MARKDOWN = "markdown"
    HTML = "html"
    PLAINTEXT = "plaintext"


class CodeOptions:
    # The language passed to monaco code editor
    language: 'CodeLanguage'
    show_line_numbers: bool
    show_mini_map: bool

    def __init__(self, language: typing.Optional['CodeLanguage'] = None, show_line_numbers: bool = False, show_mini_map: bool = False):
        self.language = language if language is not None else CodeLanguage.PLAINTEXT
        self.show_line_numbers = show_line_numbers
        self.show_mini_map = show_mini_map

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "language": self.language,
            "showLineNumbers": self.show_line_numbers,
            "showMiniMap": self.show_mini_map,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "language" in data:
            args["language"] = data["language"]
        if "showLineNumbers" in data:
            args["show_line_numbers"] = data["showLineNumbers"]
        if "showMiniMap" in data:
            args["show_mini_map"] = data["showMiniMap"]        

        return cls(**args)


class Options:
    mode: 'TextMode'
    code: typing.Optional['CodeOptions']
    content: str

    def __init__(self, mode: typing.Optional['TextMode'] = None, code: typing.Optional['CodeOptions'] = None, content: str = "# Title\n\nFor markdown syntax help: [commonmark.org/help](https://commonmark.org/help/)"):
        self.mode = mode if mode is not None else TextMode.MARKDOWN
        self.code = code
        self.content = content

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "mode": self.mode,
            "content": self.content,
        }
        if self.code is not None:
            payload["code"] = self.code
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "mode" in data:
            args["mode"] = data["mode"]
        if "code" in data:
            args["code"] = CodeOptions.from_json(data["code"])
        if "content" in data:
            args["content"] = data["content"]        

        return cls(**args)





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="text",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import preferences


class Preferences(cogbuilder.Builder[preferences.Preferences]):    
    """
    Spec defines user, team or org Grafana preferences
    swagger:model Preferences
    """
    
    _internal: preferences.Preferences

    def __init__(self):
        self._internal = preferences.Preferences()

    def build(self) -> preferences.Preferences:
        """
        Builds the object.
        """
        return self._internal    
    
    def home_dashboard_uid(self, home_dashboard_uid: str) -> typing.Self:    
        """
        UID for the home dashboard
        """
            
        self._internal.home_dashboard_uid = home_dashboard_uid
    
        return self
    
    def timezone(self, timezone: str) -> typing.Self:    
        """
        The timezone selection
        TODO: this should use the timezone defined in common
        """
            
        self._internal.timezone = timezone
    
        return self
    
    def week_start(self, week_start: str) -> typing.Self:    
        """
        day of the week (sunday, monday, etc)
        """
            
        self._internal.week_start = week_start
    
        return self
    
    def theme(self, theme: str) -> typing.Self:    
        """
        light, dark, empty is default
        """
            
        self._internal.theme = theme
    
        return self
    
    def language(self, language: str) -> typing.Self:    
        """
        Selected language (beta)
        """
            
        self._internal.language = language
    
        return self
    
    def query_history(self, query_history: cogbuilder.Builder[preferences.QueryHistoryPreference]) -> typing.Self:    
        """
        Explore query history preferences
        """
            
        query_history_resource = query_history.build()
        self._internal.query_history = query_history_resource
    
        return self
    
    def cookie_preferences(self, cookie_preferences: cogbuilder.Builder[preferences.CookiePreferences]) -> typing.Self:    
        """
        Cookie preferences
        """
            
        cookie_preferences_resource = cookie_preferences.build()
        self._internal.cookie_preferences = cookie_preferences_resource
    
        return self
    


class QueryHistoryPreference(cogbuilder.Builder[preferences.QueryHistoryPreference]):
    _internal: preferences.QueryHistoryPreference

    def __init__(self):
        self._internal = preferences.QueryHistoryPreference()

    def build(self) -> preferences.QueryHistoryPreference:
        """
        Builds the object.
        """
        return self._internal    
    
    def home_tab(self, home_tab: str) -> typing.Self:    
        """
        one of: '' | 'query' | 'starred';
        """
            
        self._internal.home_tab = home_tab
    
        return self
    


class CookiePreferences(cogbuilder.Builder[preferences.CookiePreferences]):
    _internal: preferences.CookiePreferences

    def __init__(self):
        self._internal = preferences.CookiePreferences()

    def build(self) -> preferences.CookiePreferences:
        """
        Builds the object.
        """
        return self._internal    
    
    def analytics(self, analytics: object) -> typing.Self:    
        self._internal.analytics = analytics
    
        return self
    
    def performance(self, performance: object) -> typing.Self:    
        self._internal.performance = performance
    
        return self
    
    def functional(self, functional: object) -> typing.Self:    
        self._internal.functional = functional
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..cog import variants as cogvariants
import typing
from ..models import dashboard
import enum
from ..cog import runtime as cogruntime


class Dataquery(cogvariants.Dataquery):
    """
    Manually converted from https://github.com/grafana/athena-datasource/blob/57ad707147b7a11e9a521a836d6bf9799877e0e3/src/types.ts
    """

    format_val: 'FormatOptions'
    connection_args: 'ConnectionArgs'
    table: typing.Optional[str]
    column: typing.Optional[str]
    query_id: typing.Optional[str]
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    raw_sql: str
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]

    def __init__(self, format_val: typing.Optional['FormatOptions'] = None, connection_args: typing.Optional['ConnectionArgs'] = None, table: typing.Optional[str] = None, column: typing.Optional[str] = None, query_id: typing.Optional[str] = None, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, raw_sql: str = "", datasource: typing.Optional[dashboard.DataSourceRef] = None):
        self.format_val = format_val if format_val is not None else FormatOptions.TIME_SERIES
        self.connection_args = connection_args if connection_args is not None else ConnectionArgs()
        self.table = table
        self.column = column
        self.query_id = query_id
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.raw_sql = raw_sql
        self.datasource = datasource

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "format": self.format_val,
            "connectionArgs": self.connection_args,
            "refId": self.ref_id,
            "rawSQL": self.raw_sql,
        }
        if self.table is not None:
            payload["table"] = self.table
        if self.column is not None:
            payload["column"] = self.column
        if self.query_id is not None:
            payload["queryID"] = self.query_id
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "format" in data:
            args["format_val"] = data["format"]
        if "connectionArgs" in data:
            args["connection_args"] = ConnectionArgs.from_json(data["connectionArgs"])
        if "table" in data:
            args["table"] = data["table"]
        if "column" in data:
            args["column"] = data["column"]
        if "queryID" in data:
            args["query_id"] = data["queryID"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "rawSQL" in data:
            args["raw_sql"] = data["rawSQL"]
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])        

        return cls(**args)


class FormatOptions(enum.IntEnum):
    TIME_SERIES = 0
    TABLE = 1
    LOGS = 2


class ConnectionArgs:
    region: typing.Optional[str]
    catalog: typing.Optional[str]
    database: typing.Optional[str]
    result_reuse_enabled: typing.Optional[bool]
    result_reuse_max_age_in_minutes: typing.Optional[float]

    def __init__(self, region: typing.Optional[str] = "__default", catalog: typing.Optional[str] = "__default", database: typing.Optional[str] = "__default", result_reuse_enabled: typing.Optional[bool] = False, result_reuse_max_age_in_minutes: typing.Optional[float] = 60):
        self.region = region
        self.catalog = catalog
        self.database = database
        self.result_reuse_enabled = result_reuse_enabled
        self.result_reuse_max_age_in_minutes = result_reuse_max_age_in_minutes

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.region is not None:
            payload["region"] = self.region
        if self.catalog is not None:
            payload["catalog"] = self.catalog
        if self.database is not None:
            payload["database"] = self.database
        if self.result_reuse_enabled is not None:
            payload["resultReuseEnabled"] = self.result_reuse_enabled
        if self.result_reuse_max_age_in_minutes is not None:
            payload["resultReuseMaxAgeInMinutes"] = self.result_reuse_max_age_in_minutes
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "region" in data:
            args["region"] = data["region"]
        if "catalog" in data:
            args["catalog"] = data["catalog"]
        if "database" in data:
            args["database"] = data["database"]
        if "resultReuseEnabled" in data:
            args["result_reuse_enabled"] = data["resultReuseEnabled"]
        if "resultReuseMaxAgeInMinutes" in data:
            args["result_reuse_max_age_in_minutes"] = data["resultReuseMaxAgeInMinutes"]        

        return cls(**args)


DefaultKey: typing.Literal["__default"] = "__default"





def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="grafana-athena-datasource",
        from_json_hook=Dataquery.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..models import common
import typing
from ..cog import runtime as cogruntime


class Options:
    """
    Identical to timeseries... except it does not have timezone settings
    """

    legend: common.VizLegendOptions
    tooltip: common.VizTooltipOptions
    # Name of the x field to use (defaults to first number)
    x_field: typing.Optional[str]

    def __init__(self, legend: typing.Optional[common.VizLegendOptions] = None, tooltip: typing.Optional[common.VizTooltipOptions] = None, x_field: typing.Optional[str] = None):
        self.legend = legend if legend is not None else common.VizLegendOptions()
        self.tooltip = tooltip if tooltip is not None else common.VizTooltipOptions()
        self.x_field = x_field

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "legend": self.legend,
            "tooltip": self.tooltip,
        }
        if self.x_field is not None:
            payload["xField"] = self.x_field
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "legend" in data:
            args["legend"] = common.VizLegendOptions.from_json(data["legend"])
        if "tooltip" in data:
            args["tooltip"] = common.VizTooltipOptions.from_json(data["tooltip"])
        if "xField" in data:
            args["x_field"] = data["xField"]        

        return cls(**args)


FieldConfig: typing.TypeAlias = common.GraphFieldConfig





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="trend",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=FieldConfig.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import testdata


class StreamingQuery(cogbuilder.Builder[testdata.StreamingQuery]):
    _internal: testdata.StreamingQuery

    def __init__(self):
        self._internal = testdata.StreamingQuery()

    def build(self) -> testdata.StreamingQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def type(self, type_val: typing.Literal["signal", "logs", "fetch"]) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def speed(self, speed: int) -> typing.Self:    
        self._internal.speed = speed
    
        return self
    
    def spread(self, spread: int) -> typing.Self:    
        self._internal.spread = spread
    
        return self
    
    def noise(self, noise: int) -> typing.Self:    
        self._internal.noise = noise
    
        return self
    
    def bands(self, bands: int) -> typing.Self:    
        self._internal.bands = bands
    
        return self
    
    def url(self, url: str) -> typing.Self:    
        self._internal.url = url
    
        return self
    


class PulseWaveQuery(cogbuilder.Builder[testdata.PulseWaveQuery]):
    _internal: testdata.PulseWaveQuery

    def __init__(self):
        self._internal = testdata.PulseWaveQuery()

    def build(self) -> testdata.PulseWaveQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def time_step(self, time_step: int) -> typing.Self:    
        self._internal.time_step = time_step
    
        return self
    
    def on_count(self, on_count: int) -> typing.Self:    
        self._internal.on_count = on_count
    
        return self
    
    def off_count(self, off_count: int) -> typing.Self:    
        self._internal.off_count = off_count
    
        return self
    
    def on_value(self, on_value: float) -> typing.Self:    
        self._internal.on_value = on_value
    
        return self
    
    def off_value(self, off_value: float) -> typing.Self:    
        self._internal.off_value = off_value
    
        return self
    


class SimulationQuery(cogbuilder.Builder[testdata.SimulationQuery]):
    _internal: testdata.SimulationQuery

    def __init__(self):
        self._internal = testdata.SimulationQuery()

    def build(self) -> testdata.SimulationQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def key(self, key: cogbuilder.Builder[testdata.Key]) -> typing.Self:    
        key_resource = key.build()
        self._internal.key = key_resource
    
        return self
    
    def config(self, config: dict[str, object]) -> typing.Self:    
        self._internal.config = config
    
        return self
    
    def stream(self, stream: bool) -> typing.Self:    
        self._internal.stream = stream
    
        return self
    
    def last(self, last: bool) -> typing.Self:    
        self._internal.last = last
    
        return self
    


class CSVWave(cogbuilder.Builder[testdata.CSVWave]):
    _internal: testdata.CSVWave

    def __init__(self):
        self._internal = testdata.CSVWave()

    def build(self) -> testdata.CSVWave:
        """
        Builds the object.
        """
        return self._internal    
    
    def time_step(self, time_step: int) -> typing.Self:    
        self._internal.time_step = time_step
    
        return self
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    
    def values_csv(self, values_csv: str) -> typing.Self:    
        self._internal.values_csv = values_csv
    
        return self
    
    def labels(self, labels: str) -> typing.Self:    
        self._internal.labels = labels
    
        return self
    


class NodesQuery(cogbuilder.Builder[testdata.NodesQuery]):
    _internal: testdata.NodesQuery

    def __init__(self):
        self._internal = testdata.NodesQuery()

    def build(self) -> testdata.NodesQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def type(self, type_val: typing.Literal["random", "response", "random edges"]) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def count(self, count: int) -> typing.Self:    
        self._internal.count = count
    
        return self
    


class USAQuery(cogbuilder.Builder[testdata.USAQuery]):
    _internal: testdata.USAQuery

    def __init__(self):
        self._internal = testdata.USAQuery()

    def build(self) -> testdata.USAQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def mode(self, mode: str) -> typing.Self:    
        self._internal.mode = mode
    
        return self
    
    def period(self, period: str) -> typing.Self:    
        self._internal.period = period
    
        return self
    
    def fields(self, fields: list[str]) -> typing.Self:    
        self._internal.fields = fields
    
        return self
    
    def states(self, states: list[str]) -> typing.Self:    
        self._internal.states = states
    
        return self
    


class Scenario(cogbuilder.Builder[testdata.Scenario]):    
    """
    TODO: Should this live here given it's not used in the dataquery?
    """
    
    _internal: testdata.Scenario

    def __init__(self):
        self._internal = testdata.Scenario()

    def build(self) -> testdata.Scenario:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    
    def string_input(self, string_input: str) -> typing.Self:    
        self._internal.string_input = string_input
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        self._internal.description = description
    
        return self
    
    def hide_alias_field(self, hide_alias_field: bool) -> typing.Self:    
        self._internal.hide_alias_field = hide_alias_field
    
        return self
    


class Dataquery(cogbuilder.Builder[testdata.Dataquery]):
    _internal: testdata.Dataquery

    def __init__(self):
        self._internal = testdata.Dataquery()

    def build(self) -> testdata.Dataquery:
        """
        Builds the object.
        """
        return self._internal    
    
    def alias(self, alias: str) -> typing.Self:    
        self._internal.alias = alias
    
        return self
    
    def scenario_id(self, scenario_id: testdata.TestDataQueryType) -> typing.Self:    
        self._internal.scenario_id = scenario_id
    
        return self
    
    def string_input(self, string_input: str) -> typing.Self:    
        self._internal.string_input = string_input
    
        return self
    
    def stream(self, stream: cogbuilder.Builder[testdata.StreamingQuery]) -> typing.Self:    
        stream_resource = stream.build()
        self._internal.stream = stream_resource
    
        return self
    
    def pulse_wave(self, pulse_wave: cogbuilder.Builder[testdata.PulseWaveQuery]) -> typing.Self:    
        pulse_wave_resource = pulse_wave.build()
        self._internal.pulse_wave = pulse_wave_resource
    
        return self
    
    def sim(self, sim: cogbuilder.Builder[testdata.SimulationQuery]) -> typing.Self:    
        sim_resource = sim.build()
        self._internal.sim = sim_resource
    
        return self
    
    def csv_wave(self, csv_wave: list[cogbuilder.Builder[testdata.CSVWave]]) -> typing.Self:    
        csv_wave_resources = [r1.build() for r1 in csv_wave]
        self._internal.csv_wave = csv_wave_resources
    
        return self
    
    def labels(self, labels: str) -> typing.Self:    
        self._internal.labels = labels
    
        return self
    
    def lines(self, lines: int) -> typing.Self:    
        self._internal.lines = lines
    
        return self
    
    def level_column(self, level_column: bool) -> typing.Self:    
        self._internal.level_column = level_column
    
        return self
    
    def channel(self, channel: str) -> typing.Self:    
        self._internal.channel = channel
    
        return self
    
    def nodes(self, nodes: cogbuilder.Builder[testdata.NodesQuery]) -> typing.Self:    
        nodes_resource = nodes.build()
        self._internal.nodes = nodes_resource
    
        return self
    
    def csv_file_name(self, csv_file_name: str) -> typing.Self:    
        self._internal.csv_file_name = csv_file_name
    
        return self
    
    def csv_content(self, csv_content: str) -> typing.Self:    
        self._internal.csv_content = csv_content
    
        return self
    
    def raw_frame_content(self, raw_frame_content: str) -> typing.Self:    
        self._internal.raw_frame_content = raw_frame_content
    
        return self
    
    def series_count(self, series_count: int) -> typing.Self:    
        self._internal.series_count = series_count
    
        return self
    
    def usa(self, usa: cogbuilder.Builder[testdata.USAQuery]) -> typing.Self:    
        usa_resource = usa.build()
        self._internal.usa = usa_resource
    
        return self
    
    def error_type(self, error_type: typing.Literal["server_panic", "frontend_exception", "frontend_observable"]) -> typing.Self:    
        self._internal.error_type = error_type
    
        return self
    
    def span_count(self, span_count: int) -> typing.Self:    
        self._internal.span_count = span_count
    
        return self
    
    def points(self, points: list[list[typing.Union[str, int]]]) -> typing.Self:    
        self._internal.points = points
    
        return self
    
    def drop_percent(self, drop_percent: float) -> typing.Self:    
        """
        Drop percentage (the chance we will lose a point 0-100)
        """
            
        self._internal.drop_percent = drop_percent
    
        return self
    
    def flamegraph_diff(self, flamegraph_diff: bool) -> typing.Self:    
        self._internal.flamegraph_diff = flamegraph_diff
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def datasource(self, datasource: object) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    


class Key(cogbuilder.Builder[testdata.Key]):
    _internal: testdata.Key

    def __init__(self):
        self._internal = testdata.Key()

    def build(self) -> testdata.Key:
        """
        Builds the object.
        """
        return self._internal    
    
    def type(self, type_val: str) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def tick(self, tick: float) -> typing.Self:    
        self._internal.tick = tick
    
        return self
    
    def uid(self, uid: str) -> typing.Self:    
        self._internal.uid = uid
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import librarypanel
from ..cog import variants as cogvariants
from ..models import dashboard


class LibraryPanel(cogbuilder.Builder[librarypanel.LibraryPanel]):
    _internal: librarypanel.LibraryPanel

    def __init__(self):
        self._internal = librarypanel.LibraryPanel()

    def build(self) -> librarypanel.LibraryPanel:
        """
        Builds the object.
        """
        return self._internal    
    
    def folder_uid(self, folder_uid: str) -> typing.Self:    
        """
        Folder UID
        """
            
        self._internal.folder_uid = folder_uid
    
        return self
    
    def uid(self, uid: str) -> typing.Self:    
        """
        Library element UID
        """
            
        self._internal.uid = uid
    
        return self
    
    def name(self, name: str) -> typing.Self:    
        """
        Panel name (also saved in the model)
        """
            
        if not len(name) >= 1:
            raise ValueError("len(name) must be >= 1")
        self._internal.name = name
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Panel description
        """
            
        self._internal.description = description
    
        return self
    
    def type(self, type_val: str) -> typing.Self:    
        """
        The panel type (from inside the model)
        """
            
        if not len(type_val) >= 1:
            raise ValueError("len(type_val) must be >= 1")
        self._internal.type_val = type_val
    
        return self
    
    def schema_version(self, schema_version: int) -> typing.Self:    
        """
        Dashboard version when this was saved (zero if unknown)
        """
            
        self._internal.schema_version = schema_version
    
        return self
    
    def version(self, version: int) -> typing.Self:    
        """
        panel version, incremented each time the dashboard is updated.
        """
            
        self._internal.version = version
    
        return self
    
    def model(self, model: cogbuilder.Builder[librarypanel.PanelModel]) -> typing.Self:    
        """
        TODO: should be the same panel schema defined in dashboard
        Typescript: Omit<Panel, 'gridPos' | 'id' | 'libraryPanel'>;
        """
            
        model_resource = model.build()
        self._internal.model = model_resource
    
        return self
    
    def meta(self, meta: cogbuilder.Builder[librarypanel.LibraryElementDTOMeta]) -> typing.Self:    
        """
        Object storage metadata
        """
            
        meta_resource = meta.build()
        self._internal.meta = meta_resource
    
        return self
    


class LibraryElementDTOMeta(cogbuilder.Builder[librarypanel.LibraryElementDTOMeta]):
    _internal: librarypanel.LibraryElementDTOMeta

    def __init__(self):
        self._internal = librarypanel.LibraryElementDTOMeta()

    def build(self) -> librarypanel.LibraryElementDTOMeta:
        """
        Builds the object.
        """
        return self._internal    
    
    def folder_name(self, folder_name: str) -> typing.Self:    
        self._internal.folder_name = folder_name
    
        return self
    
    def folder_uid(self, folder_uid: str) -> typing.Self:    
        self._internal.folder_uid = folder_uid
    
        return self
    
    def connected_dashboards(self, connected_dashboards: int) -> typing.Self:    
        self._internal.connected_dashboards = connected_dashboards
    
        return self
    
    def created(self, created: str) -> typing.Self:    
        self._internal.created = created
    
        return self
    
    def updated(self, updated: str) -> typing.Self:    
        self._internal.updated = updated
    
        return self
    
    def created_by(self, created_by: cogbuilder.Builder[librarypanel.LibraryElementDTOMetaUser]) -> typing.Self:    
        created_by_resource = created_by.build()
        self._internal.created_by = created_by_resource
    
        return self
    
    def updated_by(self, updated_by: cogbuilder.Builder[librarypanel.LibraryElementDTOMetaUser]) -> typing.Self:    
        updated_by_resource = updated_by.build()
        self._internal.updated_by = updated_by_resource
    
        return self
    


class LibraryElementDTOMetaUser(cogbuilder.Builder[librarypanel.LibraryElementDTOMetaUser]):
    _internal: librarypanel.LibraryElementDTOMetaUser

    def __init__(self):
        self._internal = librarypanel.LibraryElementDTOMetaUser()

    def build(self) -> librarypanel.LibraryElementDTOMetaUser:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: int) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    
    def avatar_url(self, avatar_url: str) -> typing.Self:    
        self._internal.avatar_url = avatar_url
    
        return self
    


class PanelModel(cogbuilder.Builder[librarypanel.PanelModel]):    
    """
    Dashboard panels are the basic visualization building blocks.
    """
    
    _internal: librarypanel.PanelModel

    def __init__(self):
        self._internal = librarypanel.PanelModel()

    def build(self) -> librarypanel.PanelModel:
        """
        Builds the object.
        """
        return self._internal    
    
    def type(self, type_val: str) -> typing.Self:    
        """
        The panel plugin type id. This is used to find the plugin to display the panel.
        """
            
        if not len(type_val) >= 1:
            raise ValueError("len(type_val) must be >= 1")
        self._internal.type_val = type_val
    
        return self
    
    def plugin_version(self, plugin_version: str) -> typing.Self:    
        """
        The version of the plugin that is used for this panel. This is used to find the plugin to display the panel and to migrate old panel configs.
        """
            
        self._internal.plugin_version = plugin_version
    
        return self
    
    def tags(self, tags: list[str]) -> typing.Self:    
        """
        Tags for the panel.
        """
            
        self._internal.tags = tags
    
        return self
    
    def targets(self, targets: list[cogbuilder.Builder[cogvariants.Dataquery]]) -> typing.Self:    
        """
        Depends on the panel plugin. See the plugin documentation for details.
        """
            
        targets_resources = [r1.build() for r1 in targets]
        self._internal.targets = targets_resources
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        """
        Panel title.
        """
            
        self._internal.title = title
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Panel description.
        """
            
        self._internal.description = description
    
        return self
    
    def transparent(self, transparent: bool) -> typing.Self:    
        """
        Whether to display the panel without a background.
        """
            
        self._internal.transparent = transparent
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource used in all targets.
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def links(self, links: list[cogbuilder.Builder[dashboard.DashboardLink]]) -> typing.Self:    
        """
        Panel links.
        """
            
        links_resources = [r1.build() for r1 in links]
        self._internal.links = links_resources
    
        return self
    
    def repeat(self, repeat: str) -> typing.Self:    
        """
        Name of template variable to repeat for.
        """
            
        self._internal.repeat = repeat
    
        return self
    
    def repeat_direction(self, repeat_direction: typing.Literal["h", "v"]) -> typing.Self:    
        """
        Direction to repeat in if 'repeat' is set.
        `h` for horizontal, `v` for vertical.
        """
            
        self._internal.repeat_direction = repeat_direction
    
        return self
    
    def max_per_row(self, max_per_row: float) -> typing.Self:    
        """
        Option for repeated panels that controls max items per row
        Only relevant for horizontally repeated panels
        """
            
        self._internal.max_per_row = max_per_row
    
        return self
    
    def max_data_points(self, max_data_points: float) -> typing.Self:    
        """
        The maximum number of data points that the panel queries are retrieving.
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def transformations(self, transformations: list[dashboard.DataTransformerConfig]) -> typing.Self:    
        """
        List of transformations that are applied to the panel data before rendering.
        When there are multiple transformations, Grafana applies them in the order they are listed.
        Each transformation creates a result set that then passes on to the next transformation in the processing pipeline.
        """
            
        self._internal.transformations = transformations
    
        return self
    
    def interval(self, interval: str) -> typing.Self:    
        """
        The min time interval setting defines a lower limit for the $__interval and $__interval_ms variables.
        This value must be formatted as a number followed by a valid time
        identifier like: "40s", "3d", etc.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.interval = interval
    
        return self
    
    def time_from(self, time_from: str) -> typing.Self:    
        """
        Overrides the relative time range for individual panels,
        which causes them to be different than what is selected in
        the dashboard time picker in the top-right corner of the dashboard. You can use this to show metrics from different
        time periods or days on the same dashboard.
        The value is formatted as time operation like: `now-5m` (Last 5 minutes), `now/d` (the day so far),
        `now-5d/d`(Last 5 days), `now/w` (This week so far), `now-2y/y` (Last 2 years).
        Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.time_from = time_from
    
        return self
    
    def time_shift(self, time_shift: str) -> typing.Self:    
        """
        Overrides the time range for individual panels by shifting its start and end relative to the time picker.
        For example, you can shift the time range for the panel to be two hours earlier than the dashboard time picker setting `2h`.
        Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.time_shift = time_shift
    
        return self
    
    def hide_time_override(self, hide_time_override: bool) -> typing.Self:    
        """
        Controls if the timeFrom or timeShift overrides are shown in the panel header
        """
            
        self._internal.hide_time_override = hide_time_override
    
        return self
    
    def options(self, options: object) -> typing.Self:    
        """
        It depends on the panel plugin. They are specified by the Options field in panel plugin schemas.
        """
            
        self._internal.options = options
    
        return self
    
    def field_config(self, field_config: dashboard.FieldConfigSource) -> typing.Self:    
        """
        Field options allow you to change how the data is displayed in your visualizations.
        """
            
        self._internal.field_config = field_config
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
import typing
from ..models import common
from ..cog import runtime as cogruntime


class SeriesMapping(enum.StrEnum):
    AUTO = "auto"
    MANUAL = "manual"


class ScatterShow(enum.StrEnum):
    POINTS = "points"
    LINES = "lines"
    POINTS_AND_LINES = "points+lines"


class XYDimensionConfig:
    frame: int
    x: typing.Optional[str]
    exclude: typing.Optional[list[str]]

    def __init__(self, frame: int = 0, x: typing.Optional[str] = None, exclude: typing.Optional[list[str]] = None):
        self.frame = frame
        self.x = x
        self.exclude = exclude

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "frame": self.frame,
        }
        if self.x is not None:
            payload["x"] = self.x
        if self.exclude is not None:
            payload["exclude"] = self.exclude
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "frame" in data:
            args["frame"] = data["frame"]
        if "x" in data:
            args["x"] = data["x"]
        if "exclude" in data:
            args["exclude"] = data["exclude"]        

        return cls(**args)


class FieldConfig:
    show: typing.Optional['ScatterShow']
    point_size: typing.Optional[common.ScaleDimensionConfig]
    point_color: typing.Optional[common.ColorDimensionConfig]
    line_color: typing.Optional[common.ColorDimensionConfig]
    line_width: typing.Optional[int]
    line_style: typing.Optional[common.LineStyle]
    label: typing.Optional[common.VisibilityMode]
    hide_from: typing.Optional[common.HideSeriesConfig]
    axis_placement: typing.Optional[common.AxisPlacement]
    axis_color_mode: typing.Optional[common.AxisColorMode]
    axis_label: typing.Optional[str]
    axis_width: typing.Optional[float]
    axis_soft_min: typing.Optional[float]
    axis_soft_max: typing.Optional[float]
    axis_grid_show: typing.Optional[bool]
    scale_distribution: typing.Optional[common.ScaleDistributionConfig]
    axis_centered_zero: typing.Optional[bool]
    label_value: typing.Optional[common.TextDimensionConfig]
    axis_border_show: typing.Optional[bool]

    def __init__(self, show: typing.Optional['ScatterShow'] = None, point_size: typing.Optional[common.ScaleDimensionConfig] = None, point_color: typing.Optional[common.ColorDimensionConfig] = None, line_color: typing.Optional[common.ColorDimensionConfig] = None, line_width: typing.Optional[int] = None, line_style: typing.Optional[common.LineStyle] = None, label: typing.Optional[common.VisibilityMode] = None, hide_from: typing.Optional[common.HideSeriesConfig] = None, axis_placement: typing.Optional[common.AxisPlacement] = None, axis_color_mode: typing.Optional[common.AxisColorMode] = None, axis_label: typing.Optional[str] = None, axis_width: typing.Optional[float] = None, axis_soft_min: typing.Optional[float] = None, axis_soft_max: typing.Optional[float] = None, axis_grid_show: typing.Optional[bool] = None, scale_distribution: typing.Optional[common.ScaleDistributionConfig] = None, axis_centered_zero: typing.Optional[bool] = None, label_value: typing.Optional[common.TextDimensionConfig] = None, axis_border_show: typing.Optional[bool] = None):
        self.show = show if show is not None else ScatterShow.POINTS
        self.point_size = point_size
        self.point_color = point_color
        self.line_color = line_color
        self.line_width = line_width
        self.line_style = line_style
        self.label = label if label is not None else common.VisibilityMode.AUTO
        self.hide_from = hide_from
        self.axis_placement = axis_placement
        self.axis_color_mode = axis_color_mode
        self.axis_label = axis_label
        self.axis_width = axis_width
        self.axis_soft_min = axis_soft_min
        self.axis_soft_max = axis_soft_max
        self.axis_grid_show = axis_grid_show
        self.scale_distribution = scale_distribution
        self.axis_centered_zero = axis_centered_zero
        self.label_value = label_value
        self.axis_border_show = axis_border_show

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.show is not None:
            payload["show"] = self.show
        if self.point_size is not None:
            payload["pointSize"] = self.point_size
        if self.point_color is not None:
            payload["pointColor"] = self.point_color
        if self.line_color is not None:
            payload["lineColor"] = self.line_color
        if self.line_width is not None:
            payload["lineWidth"] = self.line_width
        if self.line_style is not None:
            payload["lineStyle"] = self.line_style
        if self.label is not None:
            payload["label"] = self.label
        if self.hide_from is not None:
            payload["hideFrom"] = self.hide_from
        if self.axis_placement is not None:
            payload["axisPlacement"] = self.axis_placement
        if self.axis_color_mode is not None:
            payload["axisColorMode"] = self.axis_color_mode
        if self.axis_label is not None:
            payload["axisLabel"] = self.axis_label
        if self.axis_width is not None:
            payload["axisWidth"] = self.axis_width
        if self.axis_soft_min is not None:
            payload["axisSoftMin"] = self.axis_soft_min
        if self.axis_soft_max is not None:
            payload["axisSoftMax"] = self.axis_soft_max
        if self.axis_grid_show is not None:
            payload["axisGridShow"] = self.axis_grid_show
        if self.scale_distribution is not None:
            payload["scaleDistribution"] = self.scale_distribution
        if self.axis_centered_zero is not None:
            payload["axisCenteredZero"] = self.axis_centered_zero
        if self.label_value is not None:
            payload["labelValue"] = self.label_value
        if self.axis_border_show is not None:
            payload["axisBorderShow"] = self.axis_border_show
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "show" in data:
            args["show"] = data["show"]
        if "pointSize" in data:
            args["point_size"] = common.ScaleDimensionConfig.from_json(data["pointSize"])
        if "pointColor" in data:
            args["point_color"] = common.ColorDimensionConfig.from_json(data["pointColor"])
        if "lineColor" in data:
            args["line_color"] = common.ColorDimensionConfig.from_json(data["lineColor"])
        if "lineWidth" in data:
            args["line_width"] = data["lineWidth"]
        if "lineStyle" in data:
            args["line_style"] = common.LineStyle.from_json(data["lineStyle"])
        if "label" in data:
            args["label"] = data["label"]
        if "hideFrom" in data:
            args["hide_from"] = common.HideSeriesConfig.from_json(data["hideFrom"])
        if "axisPlacement" in data:
            args["axis_placement"] = data["axisPlacement"]
        if "axisColorMode" in data:
            args["axis_color_mode"] = data["axisColorMode"]
        if "axisLabel" in data:
            args["axis_label"] = data["axisLabel"]
        if "axisWidth" in data:
            args["axis_width"] = data["axisWidth"]
        if "axisSoftMin" in data:
            args["axis_soft_min"] = data["axisSoftMin"]
        if "axisSoftMax" in data:
            args["axis_soft_max"] = data["axisSoftMax"]
        if "axisGridShow" in data:
            args["axis_grid_show"] = data["axisGridShow"]
        if "scaleDistribution" in data:
            args["scale_distribution"] = common.ScaleDistributionConfig.from_json(data["scaleDistribution"])
        if "axisCenteredZero" in data:
            args["axis_centered_zero"] = data["axisCenteredZero"]
        if "labelValue" in data:
            args["label_value"] = common.TextDimensionConfig.from_json(data["labelValue"])
        if "axisBorderShow" in data:
            args["axis_border_show"] = data["axisBorderShow"]        

        return cls(**args)


class ScatterSeriesConfig:
    x: typing.Optional[str]
    y: typing.Optional[str]
    show: typing.Optional['ScatterShow']
    point_size: typing.Optional[common.ScaleDimensionConfig]
    point_color: typing.Optional[common.ColorDimensionConfig]
    line_color: typing.Optional[common.ColorDimensionConfig]
    line_width: typing.Optional[int]
    line_style: typing.Optional[common.LineStyle]
    label: typing.Optional[common.VisibilityMode]
    hide_from: typing.Optional[common.HideSeriesConfig]
    axis_placement: typing.Optional[common.AxisPlacement]
    axis_color_mode: typing.Optional[common.AxisColorMode]
    axis_label: typing.Optional[str]
    axis_width: typing.Optional[float]
    axis_soft_min: typing.Optional[float]
    axis_soft_max: typing.Optional[float]
    axis_grid_show: typing.Optional[bool]
    scale_distribution: typing.Optional[common.ScaleDistributionConfig]
    axis_centered_zero: typing.Optional[bool]
    name: typing.Optional[str]
    label_value: typing.Optional[common.TextDimensionConfig]
    axis_border_show: typing.Optional[bool]

    def __init__(self, x: typing.Optional[str] = None, y: typing.Optional[str] = None, show: typing.Optional['ScatterShow'] = None, point_size: typing.Optional[common.ScaleDimensionConfig] = None, point_color: typing.Optional[common.ColorDimensionConfig] = None, line_color: typing.Optional[common.ColorDimensionConfig] = None, line_width: typing.Optional[int] = None, line_style: typing.Optional[common.LineStyle] = None, label: typing.Optional[common.VisibilityMode] = None, hide_from: typing.Optional[common.HideSeriesConfig] = None, axis_placement: typing.Optional[common.AxisPlacement] = None, axis_color_mode: typing.Optional[common.AxisColorMode] = None, axis_label: typing.Optional[str] = None, axis_width: typing.Optional[float] = None, axis_soft_min: typing.Optional[float] = None, axis_soft_max: typing.Optional[float] = None, axis_grid_show: typing.Optional[bool] = None, scale_distribution: typing.Optional[common.ScaleDistributionConfig] = None, axis_centered_zero: typing.Optional[bool] = None, name: typing.Optional[str] = None, label_value: typing.Optional[common.TextDimensionConfig] = None, axis_border_show: typing.Optional[bool] = None):
        self.x = x
        self.y = y
        self.show = show if show is not None else ScatterShow.POINTS
        self.point_size = point_size
        self.point_color = point_color
        self.line_color = line_color
        self.line_width = line_width
        self.line_style = line_style
        self.label = label if label is not None else common.VisibilityMode.AUTO
        self.hide_from = hide_from
        self.axis_placement = axis_placement
        self.axis_color_mode = axis_color_mode
        self.axis_label = axis_label
        self.axis_width = axis_width
        self.axis_soft_min = axis_soft_min
        self.axis_soft_max = axis_soft_max
        self.axis_grid_show = axis_grid_show
        self.scale_distribution = scale_distribution
        self.axis_centered_zero = axis_centered_zero
        self.name = name
        self.label_value = label_value
        self.axis_border_show = axis_border_show

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.x is not None:
            payload["x"] = self.x
        if self.y is not None:
            payload["y"] = self.y
        if self.show is not None:
            payload["show"] = self.show
        if self.point_size is not None:
            payload["pointSize"] = self.point_size
        if self.point_color is not None:
            payload["pointColor"] = self.point_color
        if self.line_color is not None:
            payload["lineColor"] = self.line_color
        if self.line_width is not None:
            payload["lineWidth"] = self.line_width
        if self.line_style is not None:
            payload["lineStyle"] = self.line_style
        if self.label is not None:
            payload["label"] = self.label
        if self.hide_from is not None:
            payload["hideFrom"] = self.hide_from
        if self.axis_placement is not None:
            payload["axisPlacement"] = self.axis_placement
        if self.axis_color_mode is not None:
            payload["axisColorMode"] = self.axis_color_mode
        if self.axis_label is not None:
            payload["axisLabel"] = self.axis_label
        if self.axis_width is not None:
            payload["axisWidth"] = self.axis_width
        if self.axis_soft_min is not None:
            payload["axisSoftMin"] = self.axis_soft_min
        if self.axis_soft_max is not None:
            payload["axisSoftMax"] = self.axis_soft_max
        if self.axis_grid_show is not None:
            payload["axisGridShow"] = self.axis_grid_show
        if self.scale_distribution is not None:
            payload["scaleDistribution"] = self.scale_distribution
        if self.axis_centered_zero is not None:
            payload["axisCenteredZero"] = self.axis_centered_zero
        if self.name is not None:
            payload["name"] = self.name
        if self.label_value is not None:
            payload["labelValue"] = self.label_value
        if self.axis_border_show is not None:
            payload["axisBorderShow"] = self.axis_border_show
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "x" in data:
            args["x"] = data["x"]
        if "y" in data:
            args["y"] = data["y"]
        if "show" in data:
            args["show"] = data["show"]
        if "pointSize" in data:
            args["point_size"] = common.ScaleDimensionConfig.from_json(data["pointSize"])
        if "pointColor" in data:
            args["point_color"] = common.ColorDimensionConfig.from_json(data["pointColor"])
        if "lineColor" in data:
            args["line_color"] = common.ColorDimensionConfig.from_json(data["lineColor"])
        if "lineWidth" in data:
            args["line_width"] = data["lineWidth"]
        if "lineStyle" in data:
            args["line_style"] = common.LineStyle.from_json(data["lineStyle"])
        if "label" in data:
            args["label"] = data["label"]
        if "hideFrom" in data:
            args["hide_from"] = common.HideSeriesConfig.from_json(data["hideFrom"])
        if "axisPlacement" in data:
            args["axis_placement"] = data["axisPlacement"]
        if "axisColorMode" in data:
            args["axis_color_mode"] = data["axisColorMode"]
        if "axisLabel" in data:
            args["axis_label"] = data["axisLabel"]
        if "axisWidth" in data:
            args["axis_width"] = data["axisWidth"]
        if "axisSoftMin" in data:
            args["axis_soft_min"] = data["axisSoftMin"]
        if "axisSoftMax" in data:
            args["axis_soft_max"] = data["axisSoftMax"]
        if "axisGridShow" in data:
            args["axis_grid_show"] = data["axisGridShow"]
        if "scaleDistribution" in data:
            args["scale_distribution"] = common.ScaleDistributionConfig.from_json(data["scaleDistribution"])
        if "axisCenteredZero" in data:
            args["axis_centered_zero"] = data["axisCenteredZero"]
        if "name" in data:
            args["name"] = data["name"]
        if "labelValue" in data:
            args["label_value"] = common.TextDimensionConfig.from_json(data["labelValue"])
        if "axisBorderShow" in data:
            args["axis_border_show"] = data["axisBorderShow"]        

        return cls(**args)


class Options:
    series_mapping: typing.Optional['SeriesMapping']
    dims: 'XYDimensionConfig'
    legend: common.VizLegendOptions
    tooltip: common.VizTooltipOptions
    series: list['ScatterSeriesConfig']

    def __init__(self, series_mapping: typing.Optional['SeriesMapping'] = None, dims: typing.Optional['XYDimensionConfig'] = None, legend: typing.Optional[common.VizLegendOptions] = None, tooltip: typing.Optional[common.VizTooltipOptions] = None, series: typing.Optional[list['ScatterSeriesConfig']] = None):
        self.series_mapping = series_mapping
        self.dims = dims if dims is not None else XYDimensionConfig()
        self.legend = legend if legend is not None else common.VizLegendOptions()
        self.tooltip = tooltip if tooltip is not None else common.VizTooltipOptions()
        self.series = series if series is not None else []

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "dims": self.dims,
            "legend": self.legend,
            "tooltip": self.tooltip,
            "series": self.series,
        }
        if self.series_mapping is not None:
            payload["seriesMapping"] = self.series_mapping
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "seriesMapping" in data:
            args["series_mapping"] = data["seriesMapping"]
        if "dims" in data:
            args["dims"] = XYDimensionConfig.from_json(data["dims"])
        if "legend" in data:
            args["legend"] = common.VizLegendOptions.from_json(data["legend"])
        if "tooltip" in data:
            args["tooltip"] = common.VizTooltipOptions.from_json(data["tooltip"])
        if "series" in data:
            args["series"] = [ScatterSeriesConfig.from_json(item) for item in data["series"]]        

        return cls(**args)





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="xychart",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=FieldConfig.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import azuremonitor
from ..models import dashboard


class AzureMonitorQuery(cogbuilder.Builder[azuremonitor.AzureMonitorQuery]):
    _internal: azuremonitor.AzureMonitorQuery

    def __init__(self):
        self._internal = azuremonitor.AzureMonitorQuery()

    def build(self) -> azuremonitor.AzureMonitorQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def subscription(self, subscription: str) -> typing.Self:    
        """
        Azure subscription containing the resource(s) to be queried.
        """
            
        self._internal.subscription = subscription
    
        return self
    
    def subscriptions(self, subscriptions: list[str]) -> typing.Self:    
        """
        Subscriptions to be queried via Azure Resource Graph.
        """
            
        self._internal.subscriptions = subscriptions
    
        return self
    
    def azure_monitor(self, azure_monitor: cogbuilder.Builder[azuremonitor.AzureMetricQuery]) -> typing.Self:    
        """
        Azure Monitor Metrics sub-query properties.
        """
            
        azure_monitor_resource = azure_monitor.build()
        self._internal.azure_monitor = azure_monitor_resource
    
        return self
    
    def azure_log_analytics(self, azure_log_analytics: cogbuilder.Builder[azuremonitor.AzureLogsQuery]) -> typing.Self:    
        """
        Azure Monitor Logs sub-query properties.
        """
            
        azure_log_analytics_resource = azure_log_analytics.build()
        self._internal.azure_log_analytics = azure_log_analytics_resource
    
        return self
    
    def azure_resource_graph(self, azure_resource_graph: cogbuilder.Builder[azuremonitor.AzureResourceGraphQuery]) -> typing.Self:    
        """
        Azure Resource Graph sub-query properties.
        """
            
        azure_resource_graph_resource = azure_resource_graph.build()
        self._internal.azure_resource_graph = azure_resource_graph_resource
    
        return self
    
    def azure_traces(self, azure_traces: cogbuilder.Builder[azuremonitor.AzureTracesQuery]) -> typing.Self:    
        """
        Application Insights Traces sub-query properties.
        """
            
        azure_traces_resource = azure_traces.build()
        self._internal.azure_traces = azure_traces_resource
    
        return self
    
    def grafana_template_variable_fn(self, grafana_template_variable_fn: cogbuilder.Builder[azuremonitor.GrafanaTemplateVariableQuery]) -> typing.Self:    
        """
        @deprecated Legacy template variable support.
        """
            
        grafana_template_variable_fn_resource = grafana_template_variable_fn.build()
        self._internal.grafana_template_variable_fn = grafana_template_variable_fn_resource
    
        return self
    
    def resource_group(self, resource_group: str) -> typing.Self:    
        """
        Template variables params. These exist for backwards compatiblity with legacy template variables.
        """
            
        self._internal.resource_group = resource_group
    
        return self
    
    def namespace(self, namespace: str) -> typing.Self:    
        self._internal.namespace = namespace
    
        return self
    
    def resource(self, resource: str) -> typing.Self:    
        self._internal.resource = resource
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def region(self, region: str) -> typing.Self:    
        """
        Azure Monitor query type.
        queryType: #AzureQueryType
        """
            
        self._internal.region = region
    
        return self
    


class AzureMetricQuery(cogbuilder.Builder[azuremonitor.AzureMetricQuery]):
    _internal: azuremonitor.AzureMetricQuery

    def __init__(self):
        self._internal = azuremonitor.AzureMetricQuery()

    def build(self) -> azuremonitor.AzureMetricQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def resources(self, resources: list[cogbuilder.Builder[azuremonitor.AzureMonitorResource]]) -> typing.Self:    
        """
        Array of resource URIs to be queried.
        """
            
        resources_resources = [r1.build() for r1 in resources]
        self._internal.resources = resources_resources
    
        return self
    
    def metric_namespace(self, metric_namespace: str) -> typing.Self:    
        """
        metricNamespace is used as the resource type (or resource namespace).
        It's usually equal to the target metric namespace. e.g. microsoft.storage/storageaccounts
        Kept the name of the variable as metricNamespace to avoid backward incompatibility issues.
        """
            
        self._internal.metric_namespace = metric_namespace
    
        return self
    
    def custom_namespace(self, custom_namespace: str) -> typing.Self:    
        """
        Used as the value for the metricNamespace property when it's different from the resource namespace.
        """
            
        self._internal.custom_namespace = custom_namespace
    
        return self
    
    def metric_name(self, metric_name: str) -> typing.Self:    
        """
        The metric to query data for within the specified metricNamespace. e.g. UsedCapacity
        """
            
        self._internal.metric_name = metric_name
    
        return self
    
    def region(self, region: str) -> typing.Self:    
        """
        The Azure region containing the resource(s).
        """
            
        self._internal.region = region
    
        return self
    
    def time_grain(self, time_grain: str) -> typing.Self:    
        """
        The granularity of data points to be queried. Defaults to auto.
        """
            
        self._internal.time_grain = time_grain
    
        return self
    
    def aggregation(self, aggregation: str) -> typing.Self:    
        """
        The aggregation to be used within the query. Defaults to the primaryAggregationType defined by the metric.
        """
            
        self._internal.aggregation = aggregation
    
        return self
    
    def dimension_filters(self, dimension_filters: list[cogbuilder.Builder[azuremonitor.AzureMetricDimension]]) -> typing.Self:    
        """
        Filters to reduce the set of data returned. Dimensions that can be filtered on are defined by the metric.
        """
            
        dimension_filters_resources = [r1.build() for r1 in dimension_filters]
        self._internal.dimension_filters = dimension_filters_resources
    
        return self
    
    def top(self, top: str) -> typing.Self:    
        """
        Maximum number of records to return. Defaults to 10.
        """
            
        self._internal.top = top
    
        return self
    
    def allowed_time_grains_ms(self, allowed_time_grains_ms: list[int]) -> typing.Self:    
        """
        Time grains that are supported by the metric.
        """
            
        self._internal.allowed_time_grains_ms = allowed_time_grains_ms
    
        return self
    
    def alias(self, alias: str) -> typing.Self:    
        """
        Aliases can be set to modify the legend labels. e.g. {{ resourceGroup }}. See docs for more detail.
        """
            
        self._internal.alias = alias
    
        return self
    
    def time_grain_unit(self, time_grain_unit: str) -> typing.Self:    
        """
        @deprecated
        """
            
        self._internal.time_grain_unit = time_grain_unit
    
        return self
    
    def dimension(self, dimension: str) -> typing.Self:    
        """
        @deprecated This property was migrated to dimensionFilters and should only be accessed in the migration
        """
            
        self._internal.dimension = dimension
    
        return self
    
    def dimension_filter(self, dimension_filter: str) -> typing.Self:    
        """
        @deprecated This property was migrated to dimensionFilters and should only be accessed in the migration
        """
            
        self._internal.dimension_filter = dimension_filter
    
        return self
    
    def metric_definition(self, metric_definition: str) -> typing.Self:    
        """
        @deprecated Use metricNamespace instead
        """
            
        self._internal.metric_definition = metric_definition
    
        return self
    
    def resource_uri(self, resource_uri: str) -> typing.Self:    
        """
        @deprecated Use resourceGroup, resourceName and metricNamespace instead
        """
            
        self._internal.resource_uri = resource_uri
    
        return self
    
    def resource_group(self, resource_group: str) -> typing.Self:    
        """
        @deprecated Use resources instead
        """
            
        self._internal.resource_group = resource_group
    
        return self
    
    def resource_name(self, resource_name: str) -> typing.Self:    
        """
        @deprecated Use resources instead
        """
            
        self._internal.resource_name = resource_name
    
        return self
    


class AzureMonitorResource(cogbuilder.Builder[azuremonitor.AzureMonitorResource]):
    _internal: azuremonitor.AzureMonitorResource

    def __init__(self):
        self._internal = azuremonitor.AzureMonitorResource()

    def build(self) -> azuremonitor.AzureMonitorResource:
        """
        Builds the object.
        """
        return self._internal    
    
    def subscription(self, subscription: str) -> typing.Self:    
        self._internal.subscription = subscription
    
        return self
    
    def resource_group(self, resource_group: str) -> typing.Self:    
        self._internal.resource_group = resource_group
    
        return self
    
    def resource_name(self, resource_name: str) -> typing.Self:    
        self._internal.resource_name = resource_name
    
        return self
    
    def metric_namespace(self, metric_namespace: str) -> typing.Self:    
        self._internal.metric_namespace = metric_namespace
    
        return self
    
    def region(self, region: str) -> typing.Self:    
        self._internal.region = region
    
        return self
    


class AzureMetricDimension(cogbuilder.Builder[azuremonitor.AzureMetricDimension]):
    _internal: azuremonitor.AzureMetricDimension

    def __init__(self):
        self._internal = azuremonitor.AzureMetricDimension()

    def build(self) -> azuremonitor.AzureMetricDimension:
        """
        Builds the object.
        """
        return self._internal    
    
    def dimension(self, dimension: str) -> typing.Self:    
        """
        Name of Dimension to be filtered on.
        """
            
        self._internal.dimension = dimension
    
        return self
    
    def operator(self, operator: str) -> typing.Self:    
        """
        String denoting the filter operation. Supports 'eq' - equals,'ne' - not equals, 'sw' - starts with. Note that some dimensions may not support all operators.
        """
            
        self._internal.operator = operator
    
        return self
    
    def filters(self, filters: list[str]) -> typing.Self:    
        """
        Values to match with the filter.
        """
            
        self._internal.filters = filters
    
        return self
    
    def filter(self, filter_val: str) -> typing.Self:    
        """
        @deprecated filter is deprecated in favour of filters to support multiselect.
        """
            
        self._internal.filter_val = filter_val
    
        return self
    


class AzureLogsQuery(cogbuilder.Builder[azuremonitor.AzureLogsQuery]):    
    """
    Azure Monitor Logs sub-query properties
    """
    
    _internal: azuremonitor.AzureLogsQuery

    def __init__(self):
        self._internal = azuremonitor.AzureLogsQuery()

    def build(self) -> azuremonitor.AzureLogsQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def query(self, query: str) -> typing.Self:    
        """
        KQL query to be executed.
        """
            
        self._internal.query = query
    
        return self
    
    def result_format(self, result_format: azuremonitor.ResultFormat) -> typing.Self:    
        """
        Specifies the format results should be returned as.
        """
            
        self._internal.result_format = result_format
    
        return self
    
    def resources(self, resources: list[str]) -> typing.Self:    
        """
        Array of resource URIs to be queried.
        """
            
        self._internal.resources = resources
    
        return self
    
    def dashboard_time(self, dashboard_time: bool) -> typing.Self:    
        """
        If set to true the dashboard time range will be used as a filter for the query. Otherwise the query time ranges will be used. Defaults to false.
        """
            
        self._internal.dashboard_time = dashboard_time
    
        return self
    
    def time_column(self, time_column: str) -> typing.Self:    
        """
        If dashboardTime is set to true this value dictates which column the time filter will be applied to. Defaults to the first tables timeSpan column, the first datetime column found, or TimeGenerated
        """
            
        self._internal.time_column = time_column
    
        return self
    
    def workspace(self, workspace: str) -> typing.Self:    
        """
        Workspace ID. This was removed in Grafana 8, but remains for backwards compat.
        """
            
        self._internal.workspace = workspace
    
        return self
    
    def resource(self, resource: str) -> typing.Self:    
        """
        @deprecated Use resources instead
        """
            
        self._internal.resource = resource
    
        return self
    
    def intersect_time(self, intersect_time: bool) -> typing.Self:    
        """
        @deprecated Use dashboardTime instead
        """
            
        self._internal.intersect_time = intersect_time
    
        return self
    


class AzureResourceGraphQuery(cogbuilder.Builder[azuremonitor.AzureResourceGraphQuery]):
    _internal: azuremonitor.AzureResourceGraphQuery

    def __init__(self):
        self._internal = azuremonitor.AzureResourceGraphQuery()

    def build(self) -> azuremonitor.AzureResourceGraphQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def query(self, query: str) -> typing.Self:    
        """
        Azure Resource Graph KQL query to be executed.
        """
            
        self._internal.query = query
    
        return self
    
    def result_format(self, result_format: str) -> typing.Self:    
        """
        Specifies the format results should be returned as. Defaults to table.
        """
            
        self._internal.result_format = result_format
    
        return self
    


class AzureTracesQuery(cogbuilder.Builder[azuremonitor.AzureTracesQuery]):    
    """
    Application Insights Traces sub-query properties
    """
    
    _internal: azuremonitor.AzureTracesQuery

    def __init__(self):
        self._internal = azuremonitor.AzureTracesQuery()

    def build(self) -> azuremonitor.AzureTracesQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def result_format(self, result_format: azuremonitor.ResultFormat) -> typing.Self:    
        """
        Specifies the format results should be returned as.
        """
            
        self._internal.result_format = result_format
    
        return self
    
    def resources(self, resources: list[str]) -> typing.Self:    
        """
        Array of resource URIs to be queried.
        """
            
        self._internal.resources = resources
    
        return self
    
    def operation_id(self, operation_id: str) -> typing.Self:    
        """
        Operation ID. Used only for Traces queries.
        """
            
        self._internal.operation_id = operation_id
    
        return self
    
    def trace_types(self, trace_types: list[str]) -> typing.Self:    
        """
        Types of events to filter by.
        """
            
        self._internal.trace_types = trace_types
    
        return self
    
    def filters(self, filters: list[cogbuilder.Builder[azuremonitor.AzureTracesFilter]]) -> typing.Self:    
        """
        Filters for property values.
        """
            
        filters_resources = [r1.build() for r1 in filters]
        self._internal.filters = filters_resources
    
        return self
    
    def query(self, query: str) -> typing.Self:    
        """
        KQL query to be executed.
        """
            
        self._internal.query = query
    
        return self
    


class AzureTracesFilter(cogbuilder.Builder[azuremonitor.AzureTracesFilter]):
    _internal: azuremonitor.AzureTracesFilter

    def __init__(self):
        self._internal = azuremonitor.AzureTracesFilter()

    def build(self) -> azuremonitor.AzureTracesFilter:
        """
        Builds the object.
        """
        return self._internal    
    
    def property(self, property_val: str) -> typing.Self:    
        """
        Property name, auto-populated based on available traces.
        """
            
        self._internal.property_val = property_val
    
        return self
    
    def operation(self, operation: str) -> typing.Self:    
        """
        Comparison operator to use. Either equals or not equals.
        """
            
        self._internal.operation = operation
    
        return self
    
    def filters(self, filters: list[str]) -> typing.Self:    
        """
        Values to filter by.
        """
            
        self._internal.filters = filters
    
        return self
    


class AppInsightsMetricNameQuery(cogbuilder.Builder[azuremonitor.AppInsightsMetricNameQuery]):
    _internal: azuremonitor.AppInsightsMetricNameQuery

    def __init__(self):
        self._internal = azuremonitor.AppInsightsMetricNameQuery()        
        self._internal.kind = "AppInsightsMetricNameQuery"

    def build(self) -> azuremonitor.AppInsightsMetricNameQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    


class AppInsightsGroupByQuery(cogbuilder.Builder[azuremonitor.AppInsightsGroupByQuery]):
    _internal: azuremonitor.AppInsightsGroupByQuery

    def __init__(self):
        self._internal = azuremonitor.AppInsightsGroupByQuery()        
        self._internal.kind = "AppInsightsGroupByQuery"

    def build(self) -> azuremonitor.AppInsightsGroupByQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    
    def metric_name(self, metric_name: str) -> typing.Self:    
        self._internal.metric_name = metric_name
    
        return self
    


class SubscriptionsQuery(cogbuilder.Builder[azuremonitor.SubscriptionsQuery]):
    _internal: azuremonitor.SubscriptionsQuery

    def __init__(self):
        self._internal = azuremonitor.SubscriptionsQuery()        
        self._internal.kind = "SubscriptionsQuery"

    def build(self) -> azuremonitor.SubscriptionsQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    


class ResourceGroupsQuery(cogbuilder.Builder[azuremonitor.ResourceGroupsQuery]):
    _internal: azuremonitor.ResourceGroupsQuery

    def __init__(self):
        self._internal = azuremonitor.ResourceGroupsQuery()        
        self._internal.kind = "ResourceGroupsQuery"

    def build(self) -> azuremonitor.ResourceGroupsQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    
    def subscription(self, subscription: str) -> typing.Self:    
        self._internal.subscription = subscription
    
        return self
    


class ResourceNamesQuery(cogbuilder.Builder[azuremonitor.ResourceNamesQuery]):
    _internal: azuremonitor.ResourceNamesQuery

    def __init__(self):
        self._internal = azuremonitor.ResourceNamesQuery()        
        self._internal.kind = "ResourceNamesQuery"

    def build(self) -> azuremonitor.ResourceNamesQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    
    def subscription(self, subscription: str) -> typing.Self:    
        self._internal.subscription = subscription
    
        return self
    
    def resource_group(self, resource_group: str) -> typing.Self:    
        self._internal.resource_group = resource_group
    
        return self
    
    def metric_namespace(self, metric_namespace: str) -> typing.Self:    
        self._internal.metric_namespace = metric_namespace
    
        return self
    


class MetricNamespaceQuery(cogbuilder.Builder[azuremonitor.MetricNamespaceQuery]):
    _internal: azuremonitor.MetricNamespaceQuery

    def __init__(self):
        self._internal = azuremonitor.MetricNamespaceQuery()        
        self._internal.kind = "MetricNamespaceQuery"

    def build(self) -> azuremonitor.MetricNamespaceQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    
    def subscription(self, subscription: str) -> typing.Self:    
        self._internal.subscription = subscription
    
        return self
    
    def resource_group(self, resource_group: str) -> typing.Self:    
        self._internal.resource_group = resource_group
    
        return self
    
    def metric_namespace(self, metric_namespace: str) -> typing.Self:    
        self._internal.metric_namespace = metric_namespace
    
        return self
    
    def resource_name(self, resource_name: str) -> typing.Self:    
        self._internal.resource_name = resource_name
    
        return self
    


class MetricDefinitionsQuery(cogbuilder.Builder[azuremonitor.MetricDefinitionsQuery]):    
    """
    @deprecated Use MetricNamespaceQuery instead
    """
    
    _internal: azuremonitor.MetricDefinitionsQuery

    def __init__(self):
        self._internal = azuremonitor.MetricDefinitionsQuery()        
        self._internal.kind = "MetricDefinitionsQuery"

    def build(self) -> azuremonitor.MetricDefinitionsQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    
    def subscription(self, subscription: str) -> typing.Self:    
        self._internal.subscription = subscription
    
        return self
    
    def resource_group(self, resource_group: str) -> typing.Self:    
        self._internal.resource_group = resource_group
    
        return self
    
    def metric_namespace(self, metric_namespace: str) -> typing.Self:    
        self._internal.metric_namespace = metric_namespace
    
        return self
    
    def resource_name(self, resource_name: str) -> typing.Self:    
        self._internal.resource_name = resource_name
    
        return self
    


class MetricNamesQuery(cogbuilder.Builder[azuremonitor.MetricNamesQuery]):
    _internal: azuremonitor.MetricNamesQuery

    def __init__(self):
        self._internal = azuremonitor.MetricNamesQuery()        
        self._internal.kind = "MetricNamesQuery"

    def build(self) -> azuremonitor.MetricNamesQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    
    def subscription(self, subscription: str) -> typing.Self:    
        self._internal.subscription = subscription
    
        return self
    
    def resource_group(self, resource_group: str) -> typing.Self:    
        self._internal.resource_group = resource_group
    
        return self
    
    def resource_name(self, resource_name: str) -> typing.Self:    
        self._internal.resource_name = resource_name
    
        return self
    
    def metric_namespace(self, metric_namespace: str) -> typing.Self:    
        self._internal.metric_namespace = metric_namespace
    
        return self
    


class WorkspacesQuery(cogbuilder.Builder[azuremonitor.WorkspacesQuery]):
    _internal: azuremonitor.WorkspacesQuery

    def __init__(self):
        self._internal = azuremonitor.WorkspacesQuery()        
        self._internal.kind = "WorkspacesQuery"

    def build(self) -> azuremonitor.WorkspacesQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    
    def subscription(self, subscription: str) -> typing.Self:    
        self._internal.subscription = subscription
    
        return self
    


class UnknownQuery(cogbuilder.Builder[azuremonitor.UnknownQuery]):
    _internal: azuremonitor.UnknownQuery

    def __init__(self):
        self._internal = azuremonitor.UnknownQuery()        
        self._internal.kind = "UnknownQuery"

    def build(self) -> azuremonitor.UnknownQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    


class BaseGrafanaTemplateVariableQuery(cogbuilder.Builder[azuremonitor.BaseGrafanaTemplateVariableQuery]):
    _internal: azuremonitor.BaseGrafanaTemplateVariableQuery

    def __init__(self):
        self._internal = azuremonitor.BaseGrafanaTemplateVariableQuery()

    def build(self) -> azuremonitor.BaseGrafanaTemplateVariableQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def raw_query(self, raw_query: str) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import grafanapyroscope
from ..models import dashboard


class Dataquery(cogbuilder.Builder[grafanapyroscope.Dataquery]):
    _internal: grafanapyroscope.Dataquery

    def __init__(self):
        self._internal = grafanapyroscope.Dataquery()

    def build(self) -> grafanapyroscope.Dataquery:
        """
        Builds the object.
        """
        return self._internal    
    
    def label_selector(self, label_selector: str) -> typing.Self:    
        """
        Specifies the query label selectors.
        """
            
        self._internal.label_selector = label_selector
    
        return self
    
    def span_selector(self, span_selector: list[str]) -> typing.Self:    
        """
        Specifies the query span selectors.
        """
            
        self._internal.span_selector = span_selector
    
        return self
    
    def profile_type_id(self, profile_type_id: str) -> typing.Self:    
        """
        Specifies the type of profile to query.
        """
            
        self._internal.profile_type_id = profile_type_id
    
        return self
    
    def group_by(self, group_by: list[str]) -> typing.Self:    
        """
        Allows to group the results.
        """
            
        self._internal.group_by = group_by
    
        return self
    
    def max_nodes(self, max_nodes: int) -> typing.Self:    
        """
        Sets the maximum number of nodes in the flamegraph.
        """
            
        self._internal.max_nodes = max_nodes
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import runtime as cogruntime


class Options:
    # Comma-separated list of values used to filter alert results
    labels: str
    # Name of the alertmanager used as a source for alerts
    alertmanager: str
    # Expand all alert groups by default
    expand_all: bool

    def __init__(self, labels: str = "", alertmanager: str = "", expand_all: bool = False):
        self.labels = labels
        self.alertmanager = alertmanager
        self.expand_all = expand_all

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "labels": self.labels,
            "alertmanager": self.alertmanager,
            "expandAll": self.expand_all,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "labels" in data:
            args["labels"] = data["labels"]
        if "alertmanager" in data:
            args["alertmanager"] = data["alertmanager"]
        if "expandAll" in data:
            args["expand_all"] = data["expandAll"]        

        return cls(**args)


def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="alertGroups",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=None,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..cog import variants as cogvariants
import typing
from ..models import dashboard
import enum
from ..cog import runtime as cogruntime


class CloudMonitoringQuery(cogvariants.Dataquery):
    # A unique identifier for the query within the list of targets.
    # In server side expressions, the refId is used as a variable name to identify results.
    # By default, the UI will assign A->Z; however setting meaningful names may be useful.
    ref_id: str
    # true if query is disabled (ie should not be returned to the dashboard)
    # Note this does not always imply that the query should not be executed since
    # the results from a hidden query may be used as the input to other queries (SSE etc)
    hide: typing.Optional[bool]
    # Specify the query flavor
    # TODO make this required and give it a default
    query_type: typing.Optional[str]
    # Aliases can be set to modify the legend labels. e.g. {{metric.label.xxx}}. See docs for more detail.
    alias_by: typing.Optional[str]
    # GCM query type.
    # queryType: #QueryType
    # Time Series List sub-query properties.
    time_series_list: typing.Optional['TimeSeriesList']
    # Time Series sub-query properties.
    time_series_query: typing.Optional['TimeSeriesQuery']
    # SLO sub-query properties.
    slo_query: typing.Optional['SLOQuery']
    # PromQL sub-query properties.
    prom_ql_query: typing.Optional['PromQLQuery']
    # For mixed data sources the selected datasource is on the query level.
    # For non mixed scenarios this is undefined.
    # TODO find a better way to do this ^ that's friendly to schema
    # TODO this shouldn't be unknown but DataSourceRef | null
    datasource: typing.Optional[dashboard.DataSourceRef]
    # Time interval in milliseconds.
    interval_ms: typing.Optional[float]

    def __init__(self, ref_id: str = "", hide: typing.Optional[bool] = None, query_type: typing.Optional[str] = None, alias_by: typing.Optional[str] = None, time_series_list: typing.Optional['TimeSeriesList'] = None, time_series_query: typing.Optional['TimeSeriesQuery'] = None, slo_query: typing.Optional['SLOQuery'] = None, prom_ql_query: typing.Optional['PromQLQuery'] = None, datasource: typing.Optional[dashboard.DataSourceRef] = None, interval_ms: typing.Optional[float] = None):
        self.ref_id = ref_id
        self.hide = hide
        self.query_type = query_type
        self.alias_by = alias_by
        self.time_series_list = time_series_list
        self.time_series_query = time_series_query
        self.slo_query = slo_query
        self.prom_ql_query = prom_ql_query
        self.datasource = datasource
        self.interval_ms = interval_ms

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "refId": self.ref_id,
        }
        if self.hide is not None:
            payload["hide"] = self.hide
        if self.query_type is not None:
            payload["queryType"] = self.query_type
        if self.alias_by is not None:
            payload["aliasBy"] = self.alias_by
        if self.time_series_list is not None:
            payload["timeSeriesList"] = self.time_series_list
        if self.time_series_query is not None:
            payload["timeSeriesQuery"] = self.time_series_query
        if self.slo_query is not None:
            payload["sloQuery"] = self.slo_query
        if self.prom_ql_query is not None:
            payload["promQLQuery"] = self.prom_ql_query
        if self.datasource is not None:
            payload["datasource"] = self.datasource
        if self.interval_ms is not None:
            payload["intervalMs"] = self.interval_ms
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "hide" in data:
            args["hide"] = data["hide"]
        if "queryType" in data:
            args["query_type"] = data["queryType"]
        if "aliasBy" in data:
            args["alias_by"] = data["aliasBy"]
        if "timeSeriesList" in data:
            args["time_series_list"] = TimeSeriesList.from_json(data["timeSeriesList"])
        if "timeSeriesQuery" in data:
            args["time_series_query"] = TimeSeriesQuery.from_json(data["timeSeriesQuery"])
        if "sloQuery" in data:
            args["slo_query"] = SLOQuery.from_json(data["sloQuery"])
        if "promQLQuery" in data:
            args["prom_ql_query"] = PromQLQuery.from_json(data["promQLQuery"])
        if "datasource" in data:
            args["datasource"] = dashboard.DataSourceRef.from_json(data["datasource"])
        if "intervalMs" in data:
            args["interval_ms"] = data["intervalMs"]        

        return cls(**args)


class TimeSeriesList:
    """
    Time Series List sub-query properties.
    """

    # GCP project to execute the query against.
    project_name: str
    # Reducer applied across a set of time-series values. Defaults to REDUCE_NONE.
    cross_series_reducer: str
    # Alignment period to use when regularizing data. Defaults to cloud-monitoring-auto.
    alignment_period: typing.Optional[str]
    # Alignment function to be used. Defaults to ALIGN_MEAN.
    per_series_aligner: typing.Optional[str]
    # Array of labels to group data by.
    group_bys: typing.Optional[list[str]]
    # Array of filters to query data by. Labels that can be filtered on are defined by the metric.
    filters: typing.Optional[list[str]]
    # Data view, defaults to FULL.
    view: typing.Optional[str]
    # Annotation title.
    title: typing.Optional[str]
    # Annotation text.
    text: typing.Optional[str]
    # Only present if a preprocessor is selected. Reducer applied across a set of time-series values. Defaults to REDUCE_NONE.
    secondary_cross_series_reducer: typing.Optional[str]
    # Only present if a preprocessor is selected. Alignment period to use when regularizing data. Defaults to cloud-monitoring-auto.
    secondary_alignment_period: typing.Optional[str]
    # Only present if a preprocessor is selected. Alignment function to be used. Defaults to ALIGN_MEAN.
    secondary_per_series_aligner: typing.Optional[str]
    # Only present if a preprocessor is selected. Array of labels to group data by.
    secondary_group_bys: typing.Optional[list[str]]
    # Preprocessor is not part of the API, but is used to store the preprocessor and not affect the UI for the rest of parameters
    preprocessor: typing.Optional['PreprocessorType']

    def __init__(self, project_name: str = "", cross_series_reducer: str = "", alignment_period: typing.Optional[str] = None, per_series_aligner: typing.Optional[str] = None, group_bys: typing.Optional[list[str]] = None, filters: typing.Optional[list[str]] = None, view: typing.Optional[str] = None, title: typing.Optional[str] = None, text: typing.Optional[str] = None, secondary_cross_series_reducer: typing.Optional[str] = None, secondary_alignment_period: typing.Optional[str] = None, secondary_per_series_aligner: typing.Optional[str] = None, secondary_group_bys: typing.Optional[list[str]] = None, preprocessor: typing.Optional['PreprocessorType'] = None):
        self.project_name = project_name
        self.cross_series_reducer = cross_series_reducer
        self.alignment_period = alignment_period
        self.per_series_aligner = per_series_aligner
        self.group_bys = group_bys
        self.filters = filters
        self.view = view
        self.title = title
        self.text = text
        self.secondary_cross_series_reducer = secondary_cross_series_reducer
        self.secondary_alignment_period = secondary_alignment_period
        self.secondary_per_series_aligner = secondary_per_series_aligner
        self.secondary_group_bys = secondary_group_bys
        self.preprocessor = preprocessor

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "projectName": self.project_name,
            "crossSeriesReducer": self.cross_series_reducer,
        }
        if self.alignment_period is not None:
            payload["alignmentPeriod"] = self.alignment_period
        if self.per_series_aligner is not None:
            payload["perSeriesAligner"] = self.per_series_aligner
        if self.group_bys is not None:
            payload["groupBys"] = self.group_bys
        if self.filters is not None:
            payload["filters"] = self.filters
        if self.view is not None:
            payload["view"] = self.view
        if self.title is not None:
            payload["title"] = self.title
        if self.text is not None:
            payload["text"] = self.text
        if self.secondary_cross_series_reducer is not None:
            payload["secondaryCrossSeriesReducer"] = self.secondary_cross_series_reducer
        if self.secondary_alignment_period is not None:
            payload["secondaryAlignmentPeriod"] = self.secondary_alignment_period
        if self.secondary_per_series_aligner is not None:
            payload["secondaryPerSeriesAligner"] = self.secondary_per_series_aligner
        if self.secondary_group_bys is not None:
            payload["secondaryGroupBys"] = self.secondary_group_bys
        if self.preprocessor is not None:
            payload["preprocessor"] = self.preprocessor
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "projectName" in data:
            args["project_name"] = data["projectName"]
        if "crossSeriesReducer" in data:
            args["cross_series_reducer"] = data["crossSeriesReducer"]
        if "alignmentPeriod" in data:
            args["alignment_period"] = data["alignmentPeriod"]
        if "perSeriesAligner" in data:
            args["per_series_aligner"] = data["perSeriesAligner"]
        if "groupBys" in data:
            args["group_bys"] = data["groupBys"]
        if "filters" in data:
            args["filters"] = data["filters"]
        if "view" in data:
            args["view"] = data["view"]
        if "title" in data:
            args["title"] = data["title"]
        if "text" in data:
            args["text"] = data["text"]
        if "secondaryCrossSeriesReducer" in data:
            args["secondary_cross_series_reducer"] = data["secondaryCrossSeriesReducer"]
        if "secondaryAlignmentPeriod" in data:
            args["secondary_alignment_period"] = data["secondaryAlignmentPeriod"]
        if "secondaryPerSeriesAligner" in data:
            args["secondary_per_series_aligner"] = data["secondaryPerSeriesAligner"]
        if "secondaryGroupBys" in data:
            args["secondary_group_bys"] = data["secondaryGroupBys"]
        if "preprocessor" in data:
            args["preprocessor"] = data["preprocessor"]        

        return cls(**args)


class PreprocessorType(enum.StrEnum):
    """
    Types of pre-processor available. Defined by the metric.
    """

    NONE = "none"
    RATE = "rate"
    DELTA = "delta"


class TimeSeriesQuery:
    """
    Time Series sub-query properties.
    """

    # GCP project to execute the query against.
    project_name: str
    # MQL query to be executed.
    query: str
    # To disable the graphPeriod, it should explictly be set to 'disabled'.
    graph_period: typing.Optional[str]

    def __init__(self, project_name: str = "", query: str = "", graph_period: typing.Optional[str] = "disabled"):
        self.project_name = project_name
        self.query = query
        self.graph_period = graph_period

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "projectName": self.project_name,
            "query": self.query,
        }
        if self.graph_period is not None:
            payload["graphPeriod"] = self.graph_period
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "projectName" in data:
            args["project_name"] = data["projectName"]
        if "query" in data:
            args["query"] = data["query"]
        if "graphPeriod" in data:
            args["graph_period"] = data["graphPeriod"]        

        return cls(**args)


class SLOQuery:
    """
    SLO sub-query properties.
    """

    # GCP project to execute the query against.
    project_name: str
    # Alignment function to be used. Defaults to ALIGN_MEAN.
    per_series_aligner: typing.Optional[str]
    # Alignment period to use when regularizing data. Defaults to cloud-monitoring-auto.
    alignment_period: typing.Optional[str]
    # SLO selector.
    selector_name: str
    # ID for the service the SLO is in.
    service_id: str
    # Name for the service the SLO is in.
    service_name: str
    # ID for the SLO.
    slo_id: str
    # Name of the SLO.
    slo_name: str
    # SLO goal value.
    goal: typing.Optional[float]
    # Specific lookback period for the SLO.
    lookback_period: typing.Optional[str]

    def __init__(self, project_name: str = "", per_series_aligner: typing.Optional[str] = None, alignment_period: typing.Optional[str] = None, selector_name: str = "", service_id: str = "", service_name: str = "", slo_id: str = "", slo_name: str = "", goal: typing.Optional[float] = None, lookback_period: typing.Optional[str] = None):
        self.project_name = project_name
        self.per_series_aligner = per_series_aligner
        self.alignment_period = alignment_period
        self.selector_name = selector_name
        self.service_id = service_id
        self.service_name = service_name
        self.slo_id = slo_id
        self.slo_name = slo_name
        self.goal = goal
        self.lookback_period = lookback_period

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "projectName": self.project_name,
            "selectorName": self.selector_name,
            "serviceId": self.service_id,
            "serviceName": self.service_name,
            "sloId": self.slo_id,
            "sloName": self.slo_name,
        }
        if self.per_series_aligner is not None:
            payload["perSeriesAligner"] = self.per_series_aligner
        if self.alignment_period is not None:
            payload["alignmentPeriod"] = self.alignment_period
        if self.goal is not None:
            payload["goal"] = self.goal
        if self.lookback_period is not None:
            payload["lookbackPeriod"] = self.lookback_period
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "projectName" in data:
            args["project_name"] = data["projectName"]
        if "perSeriesAligner" in data:
            args["per_series_aligner"] = data["perSeriesAligner"]
        if "alignmentPeriod" in data:
            args["alignment_period"] = data["alignmentPeriod"]
        if "selectorName" in data:
            args["selector_name"] = data["selectorName"]
        if "serviceId" in data:
            args["service_id"] = data["serviceId"]
        if "serviceName" in data:
            args["service_name"] = data["serviceName"]
        if "sloId" in data:
            args["slo_id"] = data["sloId"]
        if "sloName" in data:
            args["slo_name"] = data["sloName"]
        if "goal" in data:
            args["goal"] = data["goal"]
        if "lookbackPeriod" in data:
            args["lookback_period"] = data["lookbackPeriod"]        

        return cls(**args)


class PromQLQuery:
    """
    PromQL sub-query properties.
    """

    # GCP project to execute the query against.
    project_name: str
    # PromQL expression/query to be executed.
    expr: str
    # PromQL min step
    step: str

    def __init__(self, project_name: str = "", expr: str = "", step: str = ""):
        self.project_name = project_name
        self.expr = expr
        self.step = step

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "projectName": self.project_name,
            "expr": self.expr,
            "step": self.step,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "projectName" in data:
            args["project_name"] = data["projectName"]
        if "expr" in data:
            args["expr"] = data["expr"]
        if "step" in data:
            args["step"] = data["step"]        

        return cls(**args)


class QueryType(enum.StrEnum):
    """
    Defines the supported queryTypes.
    """

    TIME_SERIES_LIST = "timeSeriesList"
    TIME_SERIES_QUERY = "timeSeriesQuery"
    SLO = "slo"
    ANNOTATION = "annotation"
    PROMQL = "promQL"


class MetricQuery:
    """
    @deprecated This type is for migration purposes only. Replaced by TimeSeriesList Metric sub-query properties.
    """

    # GCP project to execute the query against.
    project_name: str
    # Alignment function to be used. Defaults to ALIGN_MEAN.
    per_series_aligner: typing.Optional[str]
    # Alignment period to use when regularizing data. Defaults to cloud-monitoring-auto.
    alignment_period: typing.Optional[str]
    # Aliases can be set to modify the legend labels. e.g. {{metric.label.xxx}}. See docs for more detail.
    alias_by: typing.Optional[str]
    editor_mode: str
    metric_type: str
    # Reducer applied across a set of time-series values. Defaults to REDUCE_NONE.
    cross_series_reducer: str
    # Array of labels to group data by.
    group_bys: typing.Optional[list[str]]
    # Array of filters to query data by. Labels that can be filtered on are defined by the metric.
    filters: typing.Optional[list[str]]
    metric_kind: typing.Optional['MetricKind']
    value_type: typing.Optional[str]
    view: typing.Optional[str]
    # MQL query to be executed.
    query: str
    # Preprocessor is not part of the API, but is used to store the preprocessor and not affect the UI for the rest of parameters
    preprocessor: typing.Optional['PreprocessorType']
    # To disable the graphPeriod, it should explictly be set to 'disabled'.
    graph_period: typing.Optional[str]

    def __init__(self, project_name: str = "", per_series_aligner: typing.Optional[str] = None, alignment_period: typing.Optional[str] = None, alias_by: typing.Optional[str] = None, editor_mode: str = "", metric_type: str = "", cross_series_reducer: str = "", group_bys: typing.Optional[list[str]] = None, filters: typing.Optional[list[str]] = None, metric_kind: typing.Optional['MetricKind'] = None, value_type: typing.Optional[str] = None, view: typing.Optional[str] = None, query: str = "", preprocessor: typing.Optional['PreprocessorType'] = None, graph_period: typing.Optional[str] = "disabled"):
        self.project_name = project_name
        self.per_series_aligner = per_series_aligner
        self.alignment_period = alignment_period
        self.alias_by = alias_by
        self.editor_mode = editor_mode
        self.metric_type = metric_type
        self.cross_series_reducer = cross_series_reducer
        self.group_bys = group_bys
        self.filters = filters
        self.metric_kind = metric_kind
        self.value_type = value_type
        self.view = view
        self.query = query
        self.preprocessor = preprocessor
        self.graph_period = graph_period

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "projectName": self.project_name,
            "editorMode": self.editor_mode,
            "metricType": self.metric_type,
            "crossSeriesReducer": self.cross_series_reducer,
            "query": self.query,
        }
        if self.per_series_aligner is not None:
            payload["perSeriesAligner"] = self.per_series_aligner
        if self.alignment_period is not None:
            payload["alignmentPeriod"] = self.alignment_period
        if self.alias_by is not None:
            payload["aliasBy"] = self.alias_by
        if self.group_bys is not None:
            payload["groupBys"] = self.group_bys
        if self.filters is not None:
            payload["filters"] = self.filters
        if self.metric_kind is not None:
            payload["metricKind"] = self.metric_kind
        if self.value_type is not None:
            payload["valueType"] = self.value_type
        if self.view is not None:
            payload["view"] = self.view
        if self.preprocessor is not None:
            payload["preprocessor"] = self.preprocessor
        if self.graph_period is not None:
            payload["graphPeriod"] = self.graph_period
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "projectName" in data:
            args["project_name"] = data["projectName"]
        if "perSeriesAligner" in data:
            args["per_series_aligner"] = data["perSeriesAligner"]
        if "alignmentPeriod" in data:
            args["alignment_period"] = data["alignmentPeriod"]
        if "aliasBy" in data:
            args["alias_by"] = data["aliasBy"]
        if "editorMode" in data:
            args["editor_mode"] = data["editorMode"]
        if "metricType" in data:
            args["metric_type"] = data["metricType"]
        if "crossSeriesReducer" in data:
            args["cross_series_reducer"] = data["crossSeriesReducer"]
        if "groupBys" in data:
            args["group_bys"] = data["groupBys"]
        if "filters" in data:
            args["filters"] = data["filters"]
        if "metricKind" in data:
            args["metric_kind"] = data["metricKind"]
        if "valueType" in data:
            args["value_type"] = data["valueType"]
        if "view" in data:
            args["view"] = data["view"]
        if "query" in data:
            args["query"] = data["query"]
        if "preprocessor" in data:
            args["preprocessor"] = data["preprocessor"]
        if "graphPeriod" in data:
            args["graph_period"] = data["graphPeriod"]        

        return cls(**args)


class MetricKind(enum.StrEnum):
    METRIC_KIND_UNSPECIFIED = "METRIC_KIND_UNSPECIFIED"
    GAUGE = "GAUGE"
    DELTA = "DELTA"
    CUMULATIVE = "CUMULATIVE"


class ValueTypes(enum.StrEnum):
    VALUE_TYPE_UNSPECIFIED = "VALUE_TYPE_UNSPECIFIED"
    BOOL = "BOOL"
    INT64 = "INT64"
    DOUBLE = "DOUBLE"
    STRING = "STRING"
    DISTRIBUTION = "DISTRIBUTION"
    MONEY = "MONEY"


class AlignmentTypes(enum.StrEnum):
    ALIGN_DELTA = "ALIGN_DELTA"
    ALIGN_RATE = "ALIGN_RATE"
    ALIGN_INTERPOLATE = "ALIGN_INTERPOLATE"
    ALIGN_NEXT_OLDER = "ALIGN_NEXT_OLDER"
    ALIGN_MIN = "ALIGN_MIN"
    ALIGN_MAX = "ALIGN_MAX"
    ALIGN_MEAN = "ALIGN_MEAN"
    ALIGN_COUNT = "ALIGN_COUNT"
    ALIGN_SUM = "ALIGN_SUM"
    ALIGN_STDDEV = "ALIGN_STDDEV"
    ALIGN_COUNT_TRUE = "ALIGN_COUNT_TRUE"
    ALIGN_COUNT_FALSE = "ALIGN_COUNT_FALSE"
    ALIGN_FRACTION_TRUE = "ALIGN_FRACTION_TRUE"
    ALIGN_PERCENTILE_99 = "ALIGN_PERCENTILE_99"
    ALIGN_PERCENTILE_95 = "ALIGN_PERCENTILE_95"
    ALIGN_PERCENTILE_50 = "ALIGN_PERCENTILE_50"
    ALIGN_PERCENTILE_05 = "ALIGN_PERCENTILE_05"
    ALIGN_PERCENT_CHANGE = "ALIGN_PERCENT_CHANGE"
    ALIGN_NONE = "ALIGN_NONE"


class LegacyCloudMonitoringAnnotationQuery:
    """
    @deprecated Use TimeSeriesList instead. Legacy annotation query properties for migration purposes.
    """

    # GCP project to execute the query against.
    project_name: str
    metric_type: str
    # Query refId.
    ref_id: str
    # Array of filters to query data by. Labels that can be filtered on are defined by the metric.
    filters: list[str]
    metric_kind: 'MetricKind'
    value_type: str
    # Annotation title.
    title: str
    # Annotation text.
    text: str

    def __init__(self, project_name: str = "", metric_type: str = "", ref_id: str = "", filters: typing.Optional[list[str]] = None, metric_kind: typing.Optional['MetricKind'] = None, value_type: str = "", title: str = "", text: str = ""):
        self.project_name = project_name
        self.metric_type = metric_type
        self.ref_id = ref_id
        self.filters = filters if filters is not None else []
        self.metric_kind = metric_kind if metric_kind is not None else MetricKind.METRIC_KIND_UNSPECIFIED
        self.value_type = value_type
        self.title = title
        self.text = text

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "projectName": self.project_name,
            "metricType": self.metric_type,
            "refId": self.ref_id,
            "filters": self.filters,
            "metricKind": self.metric_kind,
            "valueType": self.value_type,
            "title": self.title,
            "text": self.text,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "projectName" in data:
            args["project_name"] = data["projectName"]
        if "metricType" in data:
            args["metric_type"] = data["metricType"]
        if "refId" in data:
            args["ref_id"] = data["refId"]
        if "filters" in data:
            args["filters"] = data["filters"]
        if "metricKind" in data:
            args["metric_kind"] = data["metricKind"]
        if "valueType" in data:
            args["value_type"] = data["valueType"]
        if "title" in data:
            args["title"] = data["title"]
        if "text" in data:
            args["text"] = data["text"]        

        return cls(**args)


class Filter:
    """
    Query filter representation.
    """

    # Filter key.
    key: str
    # Filter operator.
    operator: str
    # Filter value.
    value: str
    # Filter condition.
    condition: typing.Optional[str]

    def __init__(self, key: str = "", operator: str = "", value: str = "", condition: typing.Optional[str] = None):
        self.key = key
        self.operator = operator
        self.value = value
        self.condition = condition

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "key": self.key,
            "operator": self.operator,
            "value": self.value,
        }
        if self.condition is not None:
            payload["condition"] = self.condition
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "key" in data:
            args["key"] = data["key"]
        if "operator" in data:
            args["operator"] = data["operator"]
        if "value" in data:
            args["value"] = data["value"]
        if "condition" in data:
            args["condition"] = data["condition"]        

        return cls(**args)


class MetricFindQueryTypes(enum.StrEnum):
    PROJECTS = "projects"
    SERVICES = "services"
    DEFAULT_PROJECT = "defaultProject"
    METRIC_TYPES = "metricTypes"
    LABEL_KEYS = "labelKeys"
    LABEL_VALUES = "labelValues"
    RESOURCE_TYPES = "resourceTypes"
    AGGREGATIONS = "aggregations"
    ALIGNERS = "aligners"
    ALIGNMENT_PERIODS = "alignmentPeriods"
    SELECTORS = "selectors"
    SLO_SERVICES = "sloServices"
    SLO = "slo"





def variant_config() -> cogruntime.DataqueryConfig:
    return cogruntime.DataqueryConfig(
        identifier="cloud-monitoring",
        from_json_hook=CloudMonitoringQuery.from_json,
    )

